@@ -5,11 +5,11 @@ cat << EOF
 You forgot to uncomment a host type and flags in the Makefile. If your
 host type does not appear in the Makefile then choose one that you
 think is similar and once you have it working then add a new host type
-to the Makefile and includes.h. Please also send me the output of the
+to the Makefile and includes.h. Please also send us the output of the
 command "uname" on your system so this can be automated at some future
 time. 
 
-Andrew.Tridgell@anu.edu.au
+samba-bugs@samba.anu.edu.au
 =====================================================================
 EOF
 exit 1
