@@ -23,10 +23,6 @@
 #include "includes.h"
 #include "clitar.h"
 
-extern void setup_pkt(char *outbuf);
-extern BOOL reopen_connection(char *inbuf,char *outbuf);
-extern void do_dir(char *inbuf,char *outbuf,char *Mask,int attribute,void (*fn)(),BOOL recurse_dir);
-
 extern BOOL recurse;
 
 #define SEPARATORS " \t\n\r"
@@ -38,14 +34,23 @@ extern int Client;
 #define ATTRSET 1
 #define ATTRRESET 0
 
+static int attribute = aDIR | aSYSTEM | aHIDDEN;
+
 #ifndef CLIENT_TIMEOUT
 #define CLIENT_TIMEOUT (30*1000)
 #endif
 
 static char *tarbuf;
 static int tp, ntarf, tbufsiz;
+/* Incremental mode */
 BOOL tar_inc=False;
+/* Reset archive bit */
 BOOL tar_reset=False;
+/* Include / exclude mode (true=include, false=exclude) */
+BOOL tar_excl=True;
+char tar_type='\0';
+static char **cliplist=NULL;
+static int clipn=0;
 
 extern file_info def_finfo;
 extern BOOL lowercase;
@@ -62,8 +67,7 @@ int tarhandle;
 
 static void writetarheader();
 static void do_atar();
-void do_tar();
-void cmd_tar();
+static void do_tar();
 static void oct_it();
 static void fixtarname();
 static int dotarbuf();
@@ -72,17 +76,19 @@ static void dotareof();
 static void initarbuf();
 static int do_setrattr();
 
-/* begin smbrestore functions */
+/* restore functions */
 static long readtarheader();
 static long unoct();
 static void do_tarput();
 static void unfixtarname();
-/* end smbrestore functions */
 
 /*
- * utitlity procedures to implement tar specification including buffering
+ * tar specific utitlities
  */
 
+/****************************************************************************
+Write a tar header to buffer
+****************************************************************************/
 static void writetarheader(int f,  char *aname, int size, time_t mtime,
 		    char *amode)
 {
@@ -124,11 +130,90 @@ static void writetarheader(int f,  char *aname, int size, time_t mtime,
   (void) dotarbuf(f, hb.dummy, sizeof(hb.dummy));
 }
 
+/****************************************************************************
+Read a tar header into a hblock structure, and validate
+***************************************************************************/
+static long readtarheader(union hblock *hb, file_info *finfo, char *prefix)
+{
+  long chk, fchk;
+  int i;
+  char *jp;
 
+  /*
+   * read in a "standard" tar format header - we're not that interested
+   * in that many fields, though
+   */
+
+  /* check the checksum */
+  for (chk=0, i=sizeof(hb->dummy), jp=hb->dummy; --i>=0;) chk+=(0xFF & *jp++);
+
+  if (chk == 0)
+    return chk;
+
+  /* compensate for blanks in chksum header */
+  for (i=sizeof(hb->dbuf.chksum), jp=hb->dbuf.chksum; --i>=0;)
+    chk-=(0xFF & *jp++);
+
+  chk += ' ' * sizeof(hb->dbuf.chksum);
+
+  fchk=unoct(hb->dbuf.chksum, sizeof(hb->dbuf.chksum));
+
+  DEBUG(5, ("checksum totals chk=%d fchk=%d chksum=%s\n",
+	    chk, fchk, hb->dbuf.chksum));
+
+  if (fchk != chk)
+    {
+      DEBUG(0, ("checksums don't match %d %d\n", fchk, chk));
+      return -1;
+    }
+
+  strcpy(finfo->name, prefix);
+
+  /* use l + 1 to do the null too; do prefix - prefcnt to zap leading slash */
+  unfixtarname(finfo->name + strlen(prefix), hb->dbuf.name,
+	       strlen(hb->dbuf.name) + 1);
+
+/* can't handle links at present */
+  if (hb->dbuf.linkflag != '0') {
+    if (hb->dbuf.linkflag == 0) {
+      DEBUG(6, ("Warning: NULL link flag (gnu tar archive ?) %s\n",
+		finfo->name));
+    } else { 
+      DEBUG(0, ("this tar file appears to contain some kind of link - ignoring\n"));
+      return -2;
+    }
+  }
+    
+  if ((unoct(hb->dbuf.mode, sizeof(hb->dbuf.mode)) & S_IFDIR)
+    || (*(finfo->name+strlen(finfo->name)-1) == '\\'))
+    {
+      finfo->mode=aDIR;
+    }
+  else
+    finfo->mode=0; /* we don't care about mode at the moment, we'll
+		    * just make it a regular file */
+  /*
+   * Bug fix by richard@sj.co.uk
+   *
+   * REC: restore times correctly (as does tar)
+   * We only get the modification time of the file; set the creation time
+   * from the mod. time, and the access time to current time
+   */
+  finfo->mtime = finfo->ctime = strtol(hb->dbuf.mtime, NULL, 8);
+  finfo->atime = time(NULL);
+  finfo->size = unoct(hb->dbuf.size, sizeof(hb->dbuf.size));
+
+  return True;
+}
+
+/****************************************************************************
+Write out the tar buffer to tape or wherever
+****************************************************************************/
 static int dotarbuf(int f, char *b, int n)
 {
   int fail=1, writ=n;
 
+  /* This routine and the next one should be the only ones that do write()s */
   if (tp + n >= tbufsiz)
     {
       int diff;
@@ -155,6 +240,9 @@ static int dotarbuf(int f, char *b, int n)
   return(fail ? writ : 0);
 }
 
+/****************************************************************************
+Write a zeros to buffer / tape
+****************************************************************************/
 static void dozerobuf(int f, int n)
 {
   /* short routine just to write out n zeros to buffer -
@@ -174,6 +262,9 @@ static void dozerobuf(int f, int n)
     }
 }
 
+/****************************************************************************
+Malloc tape buffer
+****************************************************************************/
 static void initarbuf()
 {
   /* initialize tar buffer */
@@ -184,11 +275,13 @@ static void initarbuf()
   tp=0; ntarf=0;
 }
 
+/****************************************************************************
+Write two zero blocks at end of file
+****************************************************************************/
 static void dotareof(int f)
 {
   struct stat stbuf;
-  /* Two zero blocks at end of file, write out full buffer
-   * no matter what; well that's what the manual says */
+  /* Two zero blocks at end of file, write out full buffer */
 
   (void) dozerobuf(f, TBLOCK);
   (void) dozerobuf(f, TBLOCK);
@@ -199,9 +292,14 @@ static void dotareof(int f)
       return;
     }
 
+  /* Could be a pipe, in which case S_ISREG should fail,
+   * and we should write out at full size */
   if (tp > 0) write(f, tarbuf, S_ISREG(stbuf.st_mode) ? tp : tbufsiz);
 }
 
+/****************************************************************************
+(Un)mangle DOS pathname, make nonabsolute
+****************************************************************************/
 static void fixtarname(char *tptr, char *fp, int l)
 {
   /* add a '.' to start of file name, convert from ugly dos \'s in path
@@ -231,6 +329,9 @@ static void fixtarname(char *tptr, char *fp, int l)
 #endif
 }
 
+/****************************************************************************
+Convert from decimal to octal string
+****************************************************************************/
 static void oct_it (register long value, register int ndgs, register char *p)
 {
   /* Converts long to octal string, pads with leading zeros */
@@ -251,10 +352,47 @@ static void oct_it (register long value, register int ndgs, register char *p)
     p[--ndgs] = '0';
 }
 
+/****************************************************************************
+Convert from octal string to long
+***************************************************************************/
+static long unoct(char *p, int ndgs)
+{
+  long value=0;
+  /* Converts octal string to long, ignoring any non-digit */
+
+  while (--ndgs)
+    {
+      if (isdigit(*p))
+        value = (value << 3) | (long) (*p - '0');
+
+      p++;
+    }
+
+  return value;
+}
+
+/****************************************************************************
+Compare two strings in a slash insensitive way
+***************************************************************************/
+int strslashcmp(char *s1,char *s2)
+{
+  while(*s1 && *s2 &&
+	(*s1 == *s2
+	 || tolower(*s1) == tolower(*s2)
+	 || (*s1 == '\\' && *s2=='/')
+	 || (*s1 == '/' && *s2=='\\'))) {
+	  s1++; s2++;
+  }
+
+  return *s1-*s2;
+}
+
 /*
  * general smb utility functions
  */
-
+/****************************************************************************
+Set DOS file attributes
+***************************************************************************/
 static int do_setrattr(char *fname, int attr, int setit)
 {
   /*
@@ -347,14 +485,245 @@ static int do_setrattr(char *fname, int attr, int setit)
   return(True);
 }
 
-/*
- * smbclient support procedures
- */
+/****************************************************************************
+Create a file on a share
+***************************************************************************/
+static BOOL smbcreat(file_info finfo, int *fnum, char *inbuf, char *outbuf)
+{
+  char *p;
+  /* *must* be called with buffer ready malloc'ed */
+  /* open remote file */
+  
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,3,2 + strlen(finfo.name),True);
+  CVAL(outbuf,smb_com) = SMBcreate;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+  
+  SSVAL(outbuf,smb_vwv0,finfo.mode);
+  put_dos_date3(outbuf,smb_vwv1,finfo.mtime);
+  
+  p = smb_buf(outbuf);
+  *p++ = 4;      
+  strcpy(p,finfo.name);
+  
+  send_smb(Client,outbuf);
+  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
+  
+  if (CVAL(inbuf,smb_rcls) != 0)
+    {
+      DEBUG(0,("%s opening remote file %s\n",smb_errstr(inbuf),
+	       finfo.name));
+      return 0;
+    }
+  
+  *fnum = SVAL(inbuf,smb_vwv0);
+  return True;
+}
 
 /****************************************************************************
- do a tar operation on one file
+Write a file to a share
 ***************************************************************************/
+static BOOL smbwrite(int fnum, int n, int low, int high, int left,
+		     char *bufferp, char *inbuf, char *outbuf)
+{
+  /* *must* be called with buffer ready malloc'ed */
 
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,5,n + 3,True);
+  
+  memcpy(smb_buf(outbuf)+3, bufferp, n);
+  
+  set_message(outbuf,5,n + 3, False);
+  CVAL(outbuf,smb_com) = SMBwrite;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+  
+  SSVAL(outbuf,smb_vwv0,fnum);
+  SSVAL(outbuf,smb_vwv1,n);
+  SIVAL(outbuf,smb_vwv2,low);
+  SSVAL(outbuf,smb_vwv4,left);
+  CVAL(smb_buf(outbuf),0) = 1;
+  SSVAL(smb_buf(outbuf),1,n);
+
+  send_smb(Client,outbuf); 
+  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
+  
+  if (CVAL(inbuf,smb_rcls) != 0)
+    {
+      DEBUG(0,("%s writing remote file\n",smb_errstr(inbuf)));
+      return False;
+    }
+  
+  if (n != SVAL(inbuf,smb_vwv0))
+    {
+      DEBUG(0,("Error: only wrote %d bytes out of %d\n",
+	       SVAL(inbuf,smb_vwv0), n));
+      return False;
+    }
+
+  return True;
+}
+
+/****************************************************************************
+Close a file on a share
+***************************************************************************/
+static BOOL smbshut(file_info finfo, int fnum, char *inbuf, char *outbuf)
+{
+  /* *must* be called with buffer ready malloc'ed */
+
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,3,0,True);
+  CVAL(outbuf,smb_com) = SMBclose;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+  
+  SSVAL(outbuf,smb_vwv0,fnum);
+  put_dos_date3(outbuf,smb_vwv1,finfo.mtime);
+  
+  DEBUG(3,("Setting date to %s (0x%X)",
+	   asctime(LocalTime(&finfo.mtime)),
+	   finfo.mtime));
+  
+  send_smb(Client,outbuf);
+  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
+  
+  if (CVAL(inbuf,smb_rcls) != 0)
+    {
+      DEBUG(0,("%s closing remote file %s\n",smb_errstr(inbuf),
+	       finfo.name));
+      return False;
+    }
+
+  return True;
+}
+
+/****************************************************************************
+Verify existence of path on share
+***************************************************************************/
+static BOOL smbchkpath(char *fname, char *inbuf, char *outbuf)
+{
+  char *p;
+
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,0,4 + strlen(fname),True);
+  CVAL(outbuf,smb_com) = SMBchkpth;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+
+  p = smb_buf(outbuf);
+  *p++ = 4;
+  strcpy(p,fname);
+
+  send_smb(Client,outbuf);
+  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
+
+  DEBUG(5,("smbchkpath: %s\n",smb_errstr(inbuf)));
+
+  return(CVAL(inbuf,smb_rcls) == 0);
+}
+
+/****************************************************************************
+Make a directory on share
+***************************************************************************/
+static BOOL smbmkdir(char *fname, char *inbuf, char *outbuf)
+{
+  /* *must* be called with buffer ready malloc'ed */
+  char *p;
+
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,0,2 + strlen(fname),True);
+  
+  CVAL(outbuf,smb_com) = SMBmkdir;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+  
+  p = smb_buf(outbuf);
+  *p++ = 4;      
+  strcpy(p,fname);
+  
+  send_smb(Client,outbuf);
+  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
+  
+  if (CVAL(inbuf,smb_rcls) != 0)
+    {
+      DEBUG(0,("%s making remote directory %s\n",
+	       smb_errstr(inbuf),fname));
+      return(False);
+    }
+
+  return(True);
+}
+
+/****************************************************************************
+Ensure a remote path exists (make if necessary)
+***************************************************************************/
+static BOOL ensurepath(char *fname, char *inbuf, char *outbuf)
+{
+  /* *must* be called with buffer ready malloc'ed */
+  /* ensures path exists */
+
+  pstring partpath, ffname;
+  char *p=fname, *basehack;
+
+  *partpath = 0;
+
+  /* fname copied to ffname so can strtok */
+
+  strcpy(ffname, fname);
+
+  /* do a `basename' on ffname, so don't try and make file name directory */
+  if ((basehack=strrchr(ffname, '\\')) == NULL)
+    return True;
+  else
+    *basehack='\0';
+
+  p=strtok(ffname, "\\");
+
+  while (p)
+    {
+      strcat(partpath, p);
+
+      if (!smbchkpath(partpath, inbuf, outbuf)) {
+	if (!smbmkdir(partpath, inbuf, outbuf))
+	  {
+	    DEBUG(0, ("Error mkdirhiering\n"));
+	    return False;
+	  }
+	else
+	  DEBUG(3, ("mkdirhiering %s\n", partpath));
+
+      }
+
+      strcat(partpath, "\\");
+      p = strtok(NULL,"/\\");
+    }
+
+    return True;
+}
+
+int padit(char *buf, int bufsize, int padsize)
+{
+  int berr= 0;
+  int bytestowrite;
+  
+  DEBUG(0, ("Padding with %d zeros\n", padsize));
+  memset(buf, 0, bufsize);
+  while( !berr && padsize > 0 ) {
+    bytestowrite= MIN(bufsize, padsize);
+    berr = dotarbuf(tarhandle, buf, bytestowrite) != bytestowrite;
+    padsize -= bytestowrite;
+  }
+  
+  return berr;
+}
+
+/*
+ * smbclient functions
+ */
+/****************************************************************************
+append one remote file to the tar file
+***************************************************************************/
 static void do_atar(char *rname,char *lname,file_info *finfo1)
 {
   int fnum;
@@ -394,6 +763,7 @@ static void do_atar(char *rname,char *lname,file_info *finfo1)
 
   SSVAL(outbuf,smb_vwv0,0xFF);
   SSVAL(outbuf,smb_vwv2,1);
+  SSVAL(outbuf,smb_vwv3,(DENY_NONE<<4));
   SSVAL(outbuf,smb_vwv4,aSYSTEM | aHIDDEN);
   SSVAL(outbuf,smb_vwv5,aSYSTEM | aHIDDEN);
   SSVAL(outbuf,smb_vwv8,1);
@@ -469,7 +839,7 @@ static void do_atar(char *rname,char *lname,file_info *finfo1)
 	  datalen = 0;
 	}
 
-      DEBUG(2,("getting file %s of size %d bytes as a tar file %s",
+      DEBUG(1,("getting file %s of size %d bytes as a tar file %s",
 	       finfo.name,
 	       finfo.size,
 	       lname));
@@ -679,7 +1049,15 @@ static void do_atar(char *rname,char *lname,file_info *finfo1)
 	  dataptr=NULL;
 	  datalen=0;
 	}
-      
+
+       /* pad tar file with zero's if we couldn't get entire file */
+       if (nread < finfo.size)
+        {
+          DEBUG(0, ("Didn't get entire file. size=%d, nread=%d\n", finfo.size, nread));
+          if (padit(inbuf, BUFFER_SIZE, finfo.size - nread))
+              DEBUG(0,("Error writing local file\n"));
+        }
+
       /* round tar file to nearest block */
       if (finfo.size % TBLOCK)
 	dozerobuf(tarhandle, TBLOCK - (finfo.size % TBLOCK));
@@ -723,24 +1101,47 @@ static void do_atar(char *rname,char *lname,file_info *finfo1)
 	  (tp_end.tv_usec - tp_start.tv_usec)/1000;
       get_total_time_ms += this_time;
       get_total_size += finfo.size;
-      
-      DEBUG(2,("(%g kb/s) (average %g kb/s)\n",
-	       finfo.size / (1.024*this_time + 1.0e-4),
-	       get_total_size / (1.024*get_total_time_ms)));
+
+      /* Thanks to Carel-Jan Engel (ease@mail.wirehub.nl) for this one */
+      DEBUG(1,("(%g kb/s) (average %g kb/s)\n",
+	       finfo.size / MAX(0.001, (1.024*this_time)),
+	       get_total_size / MAX(0.001, (1.024*get_total_time_ms))));
     }
   
   free(inbuf);free(outbuf);
 }
 
-
-
-void do_tar(file_info *finfo)
+/****************************************************************************
+Append single file to tar file (or not)
+***************************************************************************/
+static void do_tar(file_info *finfo)
 {
   pstring rname;
 
   if (strequal(finfo->name,".") || strequal(finfo->name,".."))
     return;
 
+  /* Is it on the exclude list ? */
+  if (!tar_excl && clipn) {
+    pstring exclaim;
+
+    strcpy(exclaim, cur_dir);
+    *(exclaim+strlen(exclaim)-1)='\0';
+
+    if (clipfind(cliplist, clipn, exclaim)) {
+      DEBUG(3,("Skipping directory %s\n", exclaim));
+      return;
+    }
+
+    strcat(exclaim, "\\");
+    strcat(exclaim, finfo->name);
+
+    if (clipfind(cliplist, clipn, exclaim)) {
+      DEBUG(3,("Skipping file %s\n", exclaim));
+      return;
+    }
+  }
+
   if (finfo->mode & aDIR)
     {
       pstring saved_curdir;
@@ -765,10 +1166,9 @@ void do_tar(file_info *finfo)
        * 40755 */
       writetarheader(tarhandle, cur_dir, 0, finfo->mtime, "040755 \0");
       strcpy(mtar_mask,cur_dir);
-      strcat(mtar_mask,"*.*");
+      strcat(mtar_mask,"*");
       
-      do_dir((char *)inbuf,(char *)outbuf,
-	     mtar_mask,aSYSTEM | aHIDDEN | aDIR,do_tar,recurse);
+      do_dir((char *)inbuf,(char *)outbuf,mtar_mask,attribute,do_tar,recurse);
       strcpy(cur_dir,saved_curdir);
       free(inbuf);free(outbuf);
     }
@@ -780,84 +1180,9 @@ void do_tar(file_info *finfo)
     }
 }
 
-/*
- *
- * restore specific functions
- *
- */
-
-static long readtarheader(union hblock *hb, file_info *finfo, char *prefix)
-{
-  long chk, fchk;
-  int i;
-  char *jp;
-
-  /*
-   * read in a "standard" tar format header - we're not that interested
-   * in that many fields, though
-   */
-
-  /* check the checksum */
-  for (chk=0, i=sizeof(hb->dummy), jp=hb->dummy; --i>=0;) chk+=(0xFF & *jp++);
-
-  if (chk == 0)
-    return chk;
-
-  /* compensate for blanks in chksum header */
-  for (i=sizeof(hb->dbuf.chksum), jp=hb->dbuf.chksum; --i>=0;)
-    chk-=(0xFF & *jp++);
-
-  chk += ' ' * sizeof(hb->dbuf.chksum);
-
-  fchk=unoct(hb->dbuf.chksum, sizeof(hb->dbuf.chksum));
-
-  DEBUG(5, ("checksum totals chk=%d fchk=%d chksum=%s\n",
-	    chk, fchk, hb->dbuf.chksum));
-
-  if (fchk != chk)
-    {
-      DEBUG(0, ("checksums don't match %d %d\n", fchk, chk));
-      return -1;
-    }
-
-  strcpy(finfo->name, prefix);
-
-  /* use l + 1 to do the null too; do prefix - prefcnt to zap leading slash */
-  unfixtarname(finfo->name + strlen(prefix), hb->dbuf.name,
-	       strlen(hb->dbuf.name) + 1);
-
-/* can't handle links at present */
-  if (hb->dbuf.linkflag != '0') {
-    if (hb->dbuf.linkflag == 0) {
-      DEBUG(4, ("Warning: NULL link detected %s\n", finfo->name));
-    } else { 
-      DEBUG(0, ("this tar file appears to contain some kind of link - ignoring\n"));
-      return -2;
-    }
-  }
-    
-  if ((unoct(hb->dbuf.mode, sizeof(hb->dbuf.mode)) & S_IFDIR)
-    || (*(finfo->name+strlen(finfo->name)-1) == '\\'))
-    {
-      finfo->mode=aDIR;
-    }
-  else
-    finfo->mode=0; /* we don't care about mode at the moment, we'll
-		    * just make it a regular file */
-  /*
-   * Bug fix by richard@sj.co.uk
-   *
-   * REC: restore times correctly (as does tar)
-   * We only get the modification time of the file; set the creation time
-   * from the mod. time, and the access time to current time
-   */
-  finfo->mtime = finfo->ctime = strtol(hb->dbuf.mtime, NULL, 8);
-  finfo->atime = time(NULL);
-  finfo->size = unoct(hb->dbuf.size, sizeof(hb->dbuf.size));
-
-  return True;
-}
-
+/****************************************************************************
+Convert from UNIX to DOS file names
+***************************************************************************/
 static void unfixtarname(char *tptr, char *fp, int l)
 {
   /* remove '.' from start of file name, convert from unix /'s to
@@ -890,230 +1215,16 @@ static void unfixtarname(char *tptr, char *fp, int l)
 #endif
 }
 
-static long unoct(char *p, int ndgs)
-{
-  long value=0;
-  /* Converts octal string to long, ignoring any non-digit */
-
-  while (--ndgs)
-    {
-      if (isdigit(*p))
-        value = (value << 3) | (long) (*p - '0');
-
-      p++;
-    }
-
-  return value;
-}
-
-static BOOL smbcreat(file_info finfo, int *fnum, char *inbuf, char *outbuf)
-{
-  char *p;
-  /* *must* be called with buffer ready malloc'ed */
-  /* open remote file */
-  
-  memset(outbuf,0,smb_size);
-  set_message(outbuf,3,2 + strlen(finfo.name),True);
-  CVAL(outbuf,smb_com) = SMBcreate;
-  SSVAL(outbuf,smb_tid,cnum);
-  setup_pkt(outbuf);
-  
-  SSVAL(outbuf,smb_vwv0,finfo.mode);
-  put_dos_date3(outbuf,smb_vwv1,finfo.mtime);
-  
-  p = smb_buf(outbuf);
-  *p++ = 4;      
-  strcpy(p,finfo.name);
-  
-  send_smb(Client,outbuf);
-  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
-  
-  if (CVAL(inbuf,smb_rcls) != 0)
-    {
-      DEBUG(0,("%s opening remote file %s\n",smb_errstr(inbuf),
-	       finfo.name));
-      return 0;
-    }
-  
-  *fnum = SVAL(inbuf,smb_vwv0);
-  return True;
-}
-
-
-static BOOL smbwrite(int fnum, int n, int low, int high, int left,
-		     char *bufferp, char *inbuf, char *outbuf)
-{
-  /* *must* be called with buffer ready malloc'ed */
-
-  memset(outbuf,0,smb_size);
-  set_message(outbuf,5,n + 3,True);
-  
-  memcpy(smb_buf(outbuf)+3, bufferp, n);
-  
-  set_message(outbuf,5,n + 3, False);
-  CVAL(outbuf,smb_com) = SMBwrite;
-  SSVAL(outbuf,smb_tid,cnum);
-  setup_pkt(outbuf);
-  
-  SSVAL(outbuf,smb_vwv0,fnum);
-  SSVAL(outbuf,smb_vwv1,n);
-  SIVAL(outbuf,smb_vwv2,low);
-  SSVAL(outbuf,smb_vwv4,left);
-  CVAL(smb_buf(outbuf),0) = 1;
-  SSVAL(smb_buf(outbuf),1,n);
-
-  send_smb(Client,outbuf); 
-  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
-  
-  if (CVAL(inbuf,smb_rcls) != 0)
-    {
-      DEBUG(0,("%s writing remote file\n",smb_errstr(inbuf)));
-      return False;
-    }
-  
-  if (n != SVAL(inbuf,smb_vwv0))
-    {
-      DEBUG(0,("Error: only wrote %d bytes out of %d\n",
-	       SVAL(inbuf,smb_vwv0), n));
-      return False;
-    }
-
-  return True;
-}
-
-static BOOL smbshut(file_info finfo, int fnum, char *inbuf, char *outbuf)
-{
-  /* *must* be called with buffer ready malloc'ed */
-
-  memset(outbuf,0,smb_size);
-  set_message(outbuf,3,0,True);
-  CVAL(outbuf,smb_com) = SMBclose;
-  SSVAL(outbuf,smb_tid,cnum);
-  setup_pkt(outbuf);
-  
-  SSVAL(outbuf,smb_vwv0,fnum);
-  put_dos_date3(outbuf,smb_vwv1,finfo.mtime);
-  
-  DEBUG(3,("Setting date to %s (0x%X)",
-	   asctime(LocalTime(&finfo.mtime,GMT_TO_LOCAL)),
-	   finfo.mtime));
-  
-  send_smb(Client,outbuf);
-  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
-  
-  if (CVAL(inbuf,smb_rcls) != 0)
-    {
-      DEBUG(0,("%s closing remote file %s\n",smb_errstr(inbuf),
-	       finfo.name));
-      return False;
-    }
-
-  return True;
-}
-
-
-static BOOL smbchkpath(char *fname, char *inbuf, char *outbuf)
-{
-  char *p;
-
-  memset(outbuf,0,smb_size);
-  set_message(outbuf,0,4 + strlen(fname),True);
-  CVAL(outbuf,smb_com) = SMBchkpth;
-  SSVAL(outbuf,smb_tid,cnum);
-  setup_pkt(outbuf);
-
-  p = smb_buf(outbuf);
-  *p++ = 4;
-  strcpy(p,fname);
-
-  send_smb(Client,outbuf);
-  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
-
-  DEBUG(5,("smbchkpath: %s\n",smb_errstr(inbuf)));
-
-  return(CVAL(inbuf,smb_rcls) == 0);
-}
-
-
-static BOOL smbmkdir(char *fname, char *inbuf, char *outbuf)
-{
-  /* *must* be called with buffer ready malloc'ed */
-  char *p;
-
-  memset(outbuf,0,smb_size);
-  set_message(outbuf,0,2 + strlen(fname),True);
-  
-  CVAL(outbuf,smb_com) = SMBmkdir;
-  SSVAL(outbuf,smb_tid,cnum);
-  setup_pkt(outbuf);
-  
-  p = smb_buf(outbuf);
-  *p++ = 4;      
-  strcpy(p,fname);
-  
-  send_smb(Client,outbuf);
-  receive_smb(Client,inbuf,CLIENT_TIMEOUT);
-  
-  if (CVAL(inbuf,smb_rcls) != 0)
-    {
-      DEBUG(0,("%s making remote directory %s\n",
-	       smb_errstr(inbuf),fname));
-      return(False);
-    }
-
-  return(True);
-}
-
-static BOOL ensurepath(char *fname, char *inbuf, char *outbuf)
-{
-  /* *must* be called with buffer ready malloc'ed */
-  /* ensures path exists */
-
-  pstring partpath="", ffname;
-  char *p=fname, *basehack;
-
-  /* fname copied to ffname so can strtok */
-
-  strcpy(ffname, fname);
-
-  /* do a `basename' on ffname, so don't try and make file name directory */
-  if ((basehack=strrchr(ffname, '\\')) == NULL)
-    return True;
-  else
-    *basehack='\0';
-
-  p=strtok(ffname, "\\");
-
-  while (p)
-    {
-      strcat(partpath, p);
-
-      if (!smbchkpath(partpath, inbuf, outbuf)) {
-	if (!smbmkdir(partpath, inbuf, outbuf))
-	  {
-	    DEBUG(0, ("Error mkdirhiering\n"));
-	    return False;
-	  }
-	else
-	  DEBUG(3, ("mkdirhiering %s\n", partpath));
-
-      }
-
-      strcat(partpath, "\\");
-      p = strtok(NULL,"/\\");
-    }
-
-    return True;
-}
-
 static void do_tarput()
 {
   file_info finfo;
   int nread=0, bufread;
   char *inbuf,*outbuf; 
   int fsize=0;
-  
+  int fnum;
   struct timeval tp_start;
+  BOOL tskip=False;       /* We'll take each file as it comes */
+
   GetTimeOfDay(&tp_start);
   
   inbuf = (char *)malloc(BUFFER_SIZE + SAFETY_MARGIN);
@@ -1127,16 +1238,47 @@ static void do_tarput()
   
   /*
    * Must read in tbufsiz dollops
-   * - this is crude as should detect
    */
-  
-  while ((bufread=read(tarhandle, tarbuf, tbufsiz))) {
+
+  /* These should be the only reads in clitar.c */
+  while ((bufread=read(tarhandle, tarbuf, tbufsiz))>0) {
     char *bufferp, *endofbuffer;
-    int chunk, fnum;
-    
+    int chunk;
+
+    /* Code to handle a short read.
+     * We always need a TBLOCK full of stuff
+     */
+    if (bufread % TBLOCK) {
+      int lchunk=TBLOCK-(bufread % TBLOCK);
+      int lread;
+
+      /* It's a shorty - a short read that is */
+      DEBUG(3, ("Short read, read %d so far (need %d)\n", bufread, lchunk));
+
+      while ((lread=read(tarhandle, tarbuf+bufread, lchunk))>0) {
+	bufread+=lread;
+	if (!(lchunk-=lread)) break;
+      }
+
+      /* If we've reached EOF then that must be a short file */
+      if (lread<=0) break;
+    }
+
     bufferp=tarbuf; 
     endofbuffer=tarbuf+bufread;
-    
+
+    if (tskip) {
+      if (fsize<bufread) {
+	tskip=False;
+	bufferp+=fsize;
+	fsize=0;
+      } else {
+	if (fsize==bufread) tskip=False;
+	fsize-=bufread;
+	continue;
+      }
+    }
+
     do {
       if (!fsize)
 	{
@@ -1160,6 +1302,25 @@ static void do_tarput()
 	      break;
 	    }
 
+	  tskip=clipn
+	    && (clipfind(cliplist, clipn, finfo.name) ^ tar_excl);
+	  if (tskip) {
+	    bufferp+=TBLOCK;
+	    if (finfo.mode & aDIR)
+	      continue;
+	    else if ((fsize=finfo.size) % TBLOCK) {
+	      fsize+=TBLOCK-(fsize%TBLOCK);
+	    }
+	    if (fsize<endofbuffer-bufferp) {
+	      bufferp+=fsize;
+	      fsize=0;
+	      continue;
+	    } else {
+	      fsize-=endofbuffer-bufferp;
+	      break;
+	    }
+	  }
+
 	  if (finfo.mode & aDIR)
 	    {
 	      if (!smbchkpath(finfo.name, inbuf, outbuf)
@@ -1185,13 +1346,13 @@ static void do_tarput()
 	      free(inbuf);free(outbuf);
 	      return;
 	    }
-	  
+
 	  DEBUG(0,("restore tar file %s of size %d bytes\n",
 		   finfo.name,finfo.size));
 
 	  nread=0;
 	  if ((bufferp+=TBLOCK) >= endofbuffer) break;	  
-	} /* if nread */
+	} /* if (!fsize) */
 	
       /* write out the file in chunk sized chunks - don't
        * go past end of buffer though */
@@ -1236,20 +1397,20 @@ static void do_tarput()
 	}
     } while (bufferp < endofbuffer);
   }
-  if (ntarf) 
-    DEBUG(0, ("premature eof on tar file ?\n"));
-  else
-    DEBUG(0,("total of %d tar files restored to share\n", ntarf));
-  
+
+  DEBUG(0, ("premature eof on tar file ?\n"));
+  DEBUG(0,("total of %d tar files restored to share\n", ntarf));
+
   free(inbuf); free(outbuf);
 }
 
 /*
- * samba commands
+ * samba interactive commands
  */
 
-/* blocksize command */
-
+/****************************************************************************
+Blocksize command
+***************************************************************************/
 void cmd_block(void)
 {
   fstring buf;
@@ -1269,9 +1430,12 @@ void cmd_block(void)
     }
 
   blocksize=block;
-  DEBUG(2,("blocksize is now %d\n", blocksize));
+  DEBUG(1,("blocksize is now %d\n", blocksize));
 }
 
+/****************************************************************************
+command to set incremental / reset mode
+***************************************************************************/
 void cmd_tarmode(void)
 {
   fstring buf;
@@ -1293,16 +1457,19 @@ void cmd_tarmode(void)
 	    tar_reset ? "reset" : "noreset"));
 }
 
-/* freebie crude attrib command */
-
+/****************************************************************************
+Feeble attrib command
+***************************************************************************/
 void cmd_setmode(void)
 {
   char *q;
   fstring buf;
   pstring fname;
-  int attra[2]={0, 0};
+  int attra[2];
   int direct=1;
 
+  attra[0] = attra[1] = 0;
+
   if (!next_token(NULL,buf,NULL))
     {
       DEBUG(0, ("setmode <filename> <perm=[+|-]rsha>\n"));
@@ -1340,98 +1507,220 @@ void cmd_setmode(void)
       return;
     }
 
-DEBUG(2, ("\nperm set %d %d\n", attra[ATTRSET], attra[ATTRRESET]));
+  DEBUG(1, ("\nperm set %d %d\n", attra[ATTRSET], attra[ATTRRESET]));
   (void) do_setrattr(fname, attra[ATTRSET], ATTRSET);
   (void) do_setrattr(fname, attra[ATTRRESET], ATTRRESET);
 }
 
-/*
- * do a tar command
- */
-
+/****************************************************************************
+Principal command for creating / extracting
+***************************************************************************/
 void cmd_tar(char *inbuf, char *outbuf)
 {
-  int attribute = aDIR | aSYSTEM | aHIDDEN;
-  pstring ffname, tarmac;
   fstring buf;
-  char *p = buf;
+  char **argl;
+  int argcl;
 
   if (!next_token(NULL,buf,NULL))
     {
-      DEBUG(0,("tar [c|x] <filename>\n"));
+      DEBUG(0,("tar <c|x>[IXbga] <filename>\n"));
       return;
     }
 
-  if (*p == 'c') {
-    if (!next_token(NULL,buf,NULL))
-      {
-	DEBUG(0,("tar [c|x] <filename> [<tar mask>]\n"));
-	return;
-      }
-    
-    strcpy(ffname,p);
-    
-    if(!strcmp(ffname,"-"))
-      tarhandle = fileno(stdout);
-    else 
-      tarhandle = creat(ffname,0644);
-    
-    if (tarhandle < 0)
-      {
-	DEBUG(0,("Error opening local file %s\n",ffname));
-	return;
-      }
-    
-    initarbuf();
+  argl=toktocliplist(&argcl, NULL);
+  if (!tar_parseargs(argcl, argl, buf, 0))
+    return;
 
-    strcpy(tarmac,cur_dir);
-    if (next_token(NULL,buf,NULL))
-      do {
-	if(tarmac[strlen(tarmac)-1]!='\\')
-	  strcat(tarmac,"\\");
+  process_tar(inbuf, outbuf);
+
+  free(argl);
+}
+
+/****************************************************************************
+Command line (option) version
+***************************************************************************/
+int process_tar(char *inbuf, char *outbuf)
+{
+  initarbuf();
+  switch(tar_type) {
+  case 'x':
+    do_tarput();
+    free(tarbuf);
+    close(tarhandle);
+    break;
+  case 'r':
+  case 'c':
+    if (clipn && tar_excl) {
+      int i;
+      pstring tarmac;
+
+      for (i=0; i<clipn; i++) {
+	DEBUG(0,("arg %d = %s\n", i, cliplist[i]));
+
+	if (*(cliplist[i]+strlen(cliplist[i])-1)=='\\') {
+	  *(cliplist[i]+strlen(cliplist[i])-1)='\0';
+	}
 	
-	if (*p == '\\')
-	  strcpy(tarmac,p);
-	else
-	  strcat(tarmac,p);
-      } while (next_token(NULL,buf,NULL));
-    else
-      strcat(tarmac,"*.*");
-    
-    do_dir((char *)inbuf,(char *)outbuf,tarmac,attribute,do_tar,recurse);
+	if (strrchr(cliplist[i], '\\')) {
+	  pstring saved_dir;
+	  
+	  strcpy(saved_dir, cur_dir);
+	  
+	  if (*cliplist[i]=='\\') {
+	    strcpy(tarmac, cliplist[i]);
+	  } else {
+	    strcpy(tarmac, cur_dir);
+	    strcat(tarmac, cliplist[i]);
+	  }
+	  strcpy(cur_dir, tarmac);
+	  *(strrchr(cur_dir, '\\')+1)='\0';
+
+	  do_dir((char *)inbuf,(char *)outbuf,tarmac,attribute,do_tar,recurse);
+	  strcpy(cur_dir,saved_dir);
+	} else {
+	  strcpy(tarmac, cur_dir);
+	  strcat(tarmac, cliplist[i]);
+	  do_dir((char *)inbuf,(char *)outbuf,tarmac,attribute,do_tar,recurse);
+	}
+      }
+    } else {
+      pstring mask;
+      strcpy(mask,cur_dir);
+      strcat(mask,"\\*");
+      do_dir((char *)inbuf,(char *)outbuf,mask,attribute,do_tar,recurse);
+    }
     
     if (ntarf) dotareof(tarhandle);
     close(tarhandle);
     free(tarbuf);
     
     DEBUG(0, ("tar: dumped %d tar files\n", ntarf));
+    break;
   }
-  else if (*p=='x') {
-    if (!next_token(NULL,buf,NULL))
-      {
-	DEBUG(0,("tar [c|x] <filename> [<tar mask>]\n"));
-	return;
-      }
-    
-    strcpy(ffname,p);
-    
-    if(!strcmp(ffname,"-"))
-      tarhandle = fileno(stdin);
-    else 
-      if ((tarhandle = open(ffname,O_RDONLY)) == -1)
-	{
-	  DEBUG(0,("Error opening local file %s\n",ffname));
-	  return;
-	}
 
-    initarbuf();
-
-    do_tarput();
-
-    free(tarbuf);
-    close(tarhandle);
-  }
-  else
-    DEBUG(0,("tar [c|x] <filename>\n"));
+  return(0);
 }
 
+/****************************************************************************
+Find a token (filename) in a clip list
+***************************************************************************/
+int clipfind(char **aret, int ret, char *tok)
+{
+  if (aret==NULL) return 0;
+
+  /* ignore leading slashes or dots in token */
+  while(strchr("/\\.", *tok)) tok++;
+
+  while(ret--) {
+    char *pkey=*aret++;
+
+    /* ignore leading slashes or dots in list */
+    while(strchr("/\\.", *pkey)) pkey++;
+
+    if (!strslashcmp(pkey, tok)) return 1;
+  }
+
+  return 0;
+}
+
+/****************************************************************************
+Parse tar arguments. Sets tar_type, tar_excl, etc.
+***************************************************************************/
+int tar_parseargs(int argc, char *argv[], char *Optarg, int Optind)
+{
+  char tar_clipfl='\0';
+
+  /* Reset back to defaults - could be from interactive version 
+   * reset mode and archive mode left as they are though
+   */
+  tar_type='\0';
+  tar_excl=True;
+
+  while (*Optarg) 
+    switch(*Optarg++) {
+    case 'c':
+      tar_type='c';
+      break;
+    case 'x':
+      if (tar_type=='c') {
+	printf("Tar must be followed by only one of c or x.\n");
+	return 0;
+      }
+      tar_type='x';
+      break;
+    case 'b':
+      if (Optind>=argc || !(blocksize=atoi(argv[Optind]))) {
+	DEBUG(0,("Option b must be followed by valid blocksize\n"));
+	return 0;
+      } else {
+	Optind++;
+      }
+      break;
+    case 'g':
+      tar_inc=True;
+      break;
+    case 'N':
+      if (Optind>=argc) {
+	DEBUG(0,("Option N must be followed by valid file name\n"));
+	return 0;
+      } else {
+	struct stat stbuf;
+	extern time_t newer_than;
+	
+	if (sys_stat(argv[Optind], &stbuf) == 0) {
+	  newer_than = stbuf.st_mtime;
+	  DEBUG(1,("Getting files newer than %s",
+		   asctime(LocalTime(&newer_than))));
+	  Optind++;
+	} else {
+	  DEBUG(0,("Error setting newer-than time\n"));
+	  return 0;
+	}
+      }
+      break;
+    case 'a':
+      tar_reset=True;
+      break;
+    case 'I':
+      if (tar_clipfl) {
+	DEBUG(0,("Only one of I,X must be specified\n"));
+	return 0;
+      }
+      tar_clipfl='I';
+      break;
+    case 'X':
+      if (tar_clipfl) {
+	DEBUG(0,("Only one of I,X must be specified\n"));
+	return 0;
+      }
+      tar_clipfl='X';
+      break;
+    default:
+      DEBUG(0,("Unknown tar option\n"));
+      return 0;
+    }
+
+  if (!tar_type) {
+    printf("Option T must be followed by one of c or x.\n");
+    return 0;
+  }
+
+  tar_excl=tar_clipfl!='X';
+  if (Optind+1<argc) {
+    cliplist=argv+Optind+1;
+    clipn=argc-Optind-1;
+  }
+  if (Optind>=argc || !strcmp(argv[Optind], "-")) {
+    /* Sets tar handle to either 0 or 1, as appropriate */
+    tarhandle=(tar_type=='c');
+  } else {
+    if ((tar_type=='x' && (tarhandle = open(argv[Optind], O_RDONLY)) == -1)
+	|| (tar_type=='c' && (tarhandle=creat(argv[Optind], 0644)) < 0))
+      {
+	DEBUG(0,("Error opening local file %s\n",argv[Optind]));
+	return(0);
+      }
+  }
+
+  return 1;
+}
