@@ -40,19 +40,20 @@ static struct termio t;
 #define TCSANOW 0
 #endif
 
-int tcgetattr(int fd, struct termio *t)
+ int tcgetattr(int fd, struct termio *t)
 {
 	return ioctl(fd, TCGETA, t);
 }
 
-int tcsetattr(int fd, int flags, const struct termio *t)
+ int tcsetattr(int fd, int flags, const struct termio *t)
 {
 	if(flags & TCSAFLUSH)
 		ioctl(fd, TCFLSH, TCIOFLUSH);
 	return ioctl(fd, TCSETS, t);
 }
 
-#elif BSD_TERMIO
+#else /* SYSV_TERMIO */
+#ifdef BSD_TERMIO
 
 /* BSD TERMIO HANDLING */
 
@@ -66,17 +67,21 @@ static struct sgttyb t;
 #define TCSAFLUSH 1
 #endif
 
-int tcgetattr(int fd, struct sgttyb *t)
+#ifndef TCSANOW
+#define TCSANOW 0
+#endif
+
+ int tcgetattr(int fd, struct sgttyb *t)
 {
 	return ioctl(fd, TIOCGETP, (char *)t);
 }
 
-int tcsetattr(int fd, int flags, const struct sgttyb *t)
+ int tcsetattr(int fd, int flags, const struct sgttyb *t)
 {
 	return ioctl(fd, TIOCSETP, (char *)t);
 }
 
-#else
+#else /* BSD_TERMIO */
 
 /* POSIX TERMIO HANDLING */
 #define ECHO_IS_ON(t) ((t).c_lflag & ECHO)
@@ -84,10 +89,10 @@ int tcsetattr(int fd, int flags, const struct sgttyb *t)
 #define TURN_ECHO_ON(t) ((t).c_lflag |= ECHO)
 
 static struct termios t;
-#endif
+#endif /* BSD_TERMIO */
+#endif /* SYSV_TERMIO */
 
-char *
-getsmbpass(char *prompt)     
+char *getsmbpass(char *prompt)    
 {
   FILE *in, *out;
   int echo_off;
@@ -156,5 +161,5 @@ getsmbpass(char *prompt)
 
 #else
 
-void getsmbpasswd_dummy() {;}
+ void getsmbpasswd_dummy() {;}
 #endif
