@@ -1,3 +1,5 @@
+#ifndef _INCLUDES_H
+#define _INCLUDES_H
 /* 
    Unix SMB/Netbios implementation.
    Version 1.9.
@@ -80,6 +82,33 @@
 #define NO_RESOURCEH
 #endif
 
+#ifdef QNX
+#define NO_RESOURCEH
+#define NO_SYSMOUNTH
+#define USE_MMAP 1
+#ifdef __386__
+   #define __i386__
+#endif
+#endif
+
+#ifdef NEWS42
+#define NO_UTIMEH
+#define NO_STRFTIME
+#define NO_UTIMBUF
+#define REPLACE_MKTIME
+#define NO_TM_NAME
+#endif
+
+#ifdef OS2
+#define NO_SYSMOUNTH
+#define NO_NETIFH
+#endif
+
+#ifdef LYNX
+#define NO_SYSMOUNTH
+#endif
+
+
 #if (defined(SHADOW_PWD)||defined(OSF1_ENH_SEC)||defined(SecureWare)||defined(PWDAUTH))
 #define PASSWORD_LENGTH 16
 #endif
@@ -110,6 +139,9 @@
 #endif
 
 #include <sys/socket.h>
+#ifdef AXPROC
+#include <termio.h>
+#endif
 #include <sys/ioctl.h>
 #include <stddef.h>
 #ifdef POSIX_H
@@ -154,10 +186,6 @@
 #endif
 #endif
 
-#if USE_MMAP
-#include <sys/mman.h>
-#endif
-
 #if defined(GETPWANAM)
 #include <sys/types.h>
 #include <sys/label.h>
@@ -165,18 +193,12 @@
 #include <pwdadj.h>
 #endif
 
-#if defined(SHADOW_PWD) && !defined(NETBSD) && !defined(CONVEX)
+#if defined(SHADOW_PWD) && !defined(NETBSD) && !defined(FreeBSD) && !defined(CONVEX)
 #include <shadow.h>
 #endif
 
-/* this might be different on different systems */
-#ifdef QUOTAS 
-#include <sys/quota.h>
-#ifndef CRAY
-#include <devnm.h>
-#else
-#include <mntent.h>
-#endif
+#ifdef SYSLOG
+#include <syslog.h>
 #endif
 
 
@@ -200,12 +222,22 @@ Here come some platform specific sections
 #define USE_SETSID
 #define HAVE_BZERO
 #define HAVE_MEMMOVE
+#define USE_SIGPROCMASK
+#define USE_WAITPID
+#if 0
+/* SETFS disabled until we can check on some bug reports */
+#if _LINUX_C_LIB_VERSION_MAJOR >= 5
+#define USE_SETFS
+#endif
+#endif
 #ifdef SHADOW_PWD
+#if _LINUX_C_LIB_VERSION_MAJOR < 5
 #ifndef crypt
 #define crypt pw_encrypt
 #endif
 #endif
 #endif
+#endif
 
 #ifdef SUNOS4
 #define SIGNAL_CAST (void (*)(int))
@@ -217,6 +249,7 @@ Here come some platform specific sections
 #include <errno.h>
 #include <sys/wait.h>
 #include <signal.h>
+/* #include <termios.h> */
 #ifdef sun386
 #define NO_STRFTIME
 #define NO_UTIMBUF
@@ -226,8 +259,21 @@ typedef unsigned short mode_t;
 #include <utime.h>
 #define NO_STRERROR
 #endif
+#ifndef REPLACE_GETPASS
 #define REPLACE_GETPASS
 #endif
+#ifndef BSD_TERMIO
+#define BSD_TERMIO
+#endif
+#ifndef USE_SIGPROCMASK
+#define USE_SIGPROCMASK
+#endif
+#ifndef USE_WAITPID
+#define USE_WAITPID
+#endif
+/* SunOS doesn't have POSIX atexit */
+#define atexit on_exit
+#endif
 
 
 #ifdef SUNOS5
@@ -244,7 +290,10 @@ typedef unsigned short mode_t;
 #include <string.h>
 #include <arpa/inet.h>
 #include <rpcsvc/ypclnt.h>
-#include <crypt.h> 
+#include <termios.h>
+#ifndef USE_LIBDES
+#include <crypt.h>
+#endif /* USE_LIBDES */
 extern int gettimeofday (struct timeval *, void *);
 extern int gethostname (char *name, int namelen);
 extern int innetgr (const char *, const char *, const char *, const char *);
@@ -259,6 +308,7 @@ extern int innetgr (const char *, const char *, const char *, const char *);
 #define USE_GETCWD
 #define USE_SETSID
 #define REPLACE_GETPASS
+#define USE_SIGPROCMASK
 #endif
 
 
@@ -276,6 +326,7 @@ char *getwd(char *);
 #define SIGNAL_CAST (void(*)(int))
 #endif
 #define USE_DIRECT
+#define USE_WAITPID
 #endif
 
 #ifdef SGI
@@ -353,12 +404,15 @@ extern struct passwd *getpwnam();
 #include <sys/filio.h>
 #include <fcntl.h>
 #include <sys/sockio.h>
+#include <netinet/tcp.h>
+#include <stropts.h>
 #include <termios.h>
 #define SYSV
 #define USE_WAITPID
 #define SIGNAL_CAST (void (*)(int))
 #define USE_STATVFS
 #define USE_GETCWD
+#define USE_SETSID
 #endif
 
 
@@ -381,6 +435,7 @@ char *mktemp(char *); /* No standard include */
 #include <prot.h>
 #include <unistd.h>
 #define PASSWORD_LENGTH 16
+#define NEED_AUTH_PARAMETERS
 #endif  /* OSF1_ENH_SEC */
 #endif
 
@@ -418,6 +473,7 @@ char *mktemp(char *); /* No standard include */
 #define NO_GETSPNAM
 #define SIGNAL_CAST (void (*)())
 #define USE_DIRECT
+#define REPLACE_INNETGR
 #endif 
 
 
@@ -428,7 +484,16 @@ char *mktemp(char *); /* No standard include */
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #define SIGNAL_CAST (void (*)())
+#define USE_SETVBUF
+#define USE_SETSID
+#define USE_GETCWD
+#define USE_WAITPID
 #define USE_DIRECT
+#define HAVE_MEMMOVE
+#define HAVE_BZERO
+#define HAVE_GETTIMEOFDAY
+#define HAVE_PATHCONF
+#define HAVE_GETGRNAM 1
 #endif 
 
 
@@ -443,10 +508,14 @@ char *mktemp(char *); /* No standard include */
 #include <sys/id.h>
 #include <sys/priv.h>
 #include <netinet/tcp.h>
+#include <locale.h>
 #define SYSV
 #define USE_WAITPID
+#define USE_SIGBLOCK
 #define SIGNAL_CAST (void (*)())
 #define DEFAULT_PRINTING PRINT_AIX
+/* we undef this because sys/param.h is broken in aix. uggh. */
+#undef MAXHOSTNAMELEN
 #endif
 
 
@@ -458,6 +527,11 @@ char *mktemp(char *); /* No standard include */
 #include <sys/types.h>
 #include <sys/termios.h>
 #include <netinet/tcp.h>
+#ifdef HPUX_10_TRUSTED
+#include <hpsecurity.h>
+#include <prot.h>
+#define NEED_AUTH_PARAMETERS
+#endif
 #define SIGNAL_CAST (void (*)(__harg))
 #define SELECT_CAST (int *)
 #define SYSV
@@ -467,6 +541,7 @@ char *mktemp(char *); /* No standard include */
 #define USE_SETSID
 #define USE_SETRES
 #define DEFAULT_PRINTING PRINT_HPUX
+#define SIGCLD_IGNORE
 #endif
 
 
@@ -487,21 +562,36 @@ char *mktemp(char *); /* No standard include */
 #define NO_EID
 #define STATFS4
 #define USE_DIRECT
+#ifdef PTX4
+#undef USE_DIRECT
+#endif
 #endif
 
 
-#ifdef NEXT
-#include <strings.h>
+
+#ifdef SEQUENT_PTX4
+#include <string.h>
 #include <sys/dir.h>
 #include <dirent.h>
+#include <sys/statfs.h>
+#include <sys/statvfs.h>
 #include <sys/vfs.h>
-#define bzero(b,len) memset(b,0,len)
-#include <libc.h>
-#define NOSTRDUP
+#include <fcntl.h>
+#include <sys/sockio.h>
+#include <netinet/tcp.h>
+#include <stropts.h>
+#include <termios.h>
+#define SYSV
 #define USE_WAITPID
-#define NO_STRFTIME
+#define SIGNAL_CAST (void (*)(int))
+#define USE_STATVFS
 #define USE_GETCWD
-#define WAIT3_CAST1 (union wait *)
+#ifndef seteuid
+#define seteuid(uid) setreuid(-1,uid)
+#endif
+#ifndef setegid
+#define setegid(gid) setregid(-1,gid)
+#endif
 #endif
 
 
@@ -530,6 +620,12 @@ char *mktemp(char *); /* No standard include */
 #define NOSTRDUP
 #define USE_DIRECT
 #define mode_t int
+#define GID_TYPE int
+#define gid_t int
+#define pid_t int
+#define SIGNAL_CAST (void (*)(int))
+#define WAIT3_CAST1 (union wait *)
+#define HAVE_GMTOFF
 #endif
 
 
@@ -558,6 +654,11 @@ char *mktemp(char *); /* No standard include */
 #include <fcntl.h>
 #include <sys/statfs.h>
 #include <sys/stropts.h>
+#include <limits.h>
+#include <locale.h>
+#ifdef EVEREST
+#include <unistd.h> 
+#endif
 #ifdef NETGROUP
 #include <rpcsvc/ypclnt.h>
 #endif
@@ -567,7 +668,9 @@ char *mktemp(char *); /* No standard include */
 #include <prot.h>
 #define crypt bigcrypt
 #endif
-#define ftruncate(f,l) syscall(0x0a28,f,l)
+#ifndef EVEREST
+ #define ftruncate(f,l) syscall(0x0a28,f,l)
+#endif 
 #define SIGNAL_CAST (void (*)(int))
 #define USE_WAITPID
 #define USE_GETCWD
@@ -575,11 +678,15 @@ char *mktemp(char *); /* No standard include */
 #ifdef SCO3_2_2
 #define NO_EID
 #else
+#ifndef EVEREST
 #define USE_IFREQ
 #endif
+#endif
 #define STATFS4
 #define NO_FSYNC
+#ifndef EVEREST
 #define NO_INITGROUPS
+#endif
 #define HAVE_PATHCONF
 #define NO_GETRLIMIT
 #endif
@@ -676,7 +783,6 @@ char *strdup (char *);
 #endif /* DNIX */
 
 #ifdef CONVEX
-#define SIGNAL_CAST (void (*)(int))
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <dirent.h>
@@ -730,9 +836,7 @@ extern char *getpass(char *);
 #define        gid_t           int
 #define        mode_t          int
 #define        ptrdiff_t       int
-
 #define HAVE_GETGRNAM  0
-
 #define NO_EID
 #define NO_FSYNC
 #define        NO_FTRUNCATE
@@ -752,22 +856,137 @@ extern char *getpass(char *);
 #define        USE_GETCWD
 #endif
 
+#ifdef QNX
+#define STATFS4
+#include <sys/statfs.h>
+#include <sys/select.h>
+#include <signal.h>
+#include <sys/dir.h>
+#define SIGNAL_CAST (void (*)())
+#define USE_WAITPID
+#define NO_INITGROUPS
+#define NO_SETGROUPS
+#define HAVE_TIMEZONE
+#define USE_GETCWD
+#define USE_SETSID
+#define HAVE_FCNTL_LOCK 1
+#define DEFAULT_PRINTING PRINT_QNX
+#endif
+
+
+#ifdef NEWS42
+#include <string.h>
+#include <dirent.h>
+#include <sys/vfs.h>
+#include <sys/timeb.h>
+typedef int mode_t;
+#endif
+
+#ifdef OS2
+#include <dirent.h>
+#include <sys/statfs.h>
+#include <string.h>
+#include <limits.h>
+#define SIGNAL_CAST (void (*)())
+#define HAVE_FCNTL_LOCK 0
+#define USE_WAITPID
+#define NO_GET_BROADCAST
+#define NO_EID
+#define NO_SETGROUPS
+#define NO_INITGROUPS
+#define NO_CRYPT
+#define NO_STATFS
+#define NO_CHROOT
+#define NO_CHOWN
+#define strcasecmp stricmp
+#define strncasecmp strnicmp
+#endif
+
+
+#ifdef LYNX
+#define SIGNAL_CAST (void (*)())
+#define WAIT3_CAST1 (union wait *)
+#define STATFS4
+#include <fcntl.h>
+#include <resource.h>
+#include <stat.h>
+#include <string.h>
+#include <dirent.h>
+#include <sys/statfs.h>
+#define USE_GETCWD
+#define USE_GETSID
+#endif
+
+
+#ifdef BOS
+#define SIGNAL_CAST (void (*)(int))
+#include <string.h>
+#include <sys/dir.h>
+#include <sys/select.h>
+#include <dirent.h>
+#include <fcntl.h>
+#include <signal.h>
+#include <sys/statfs.h>
+#include <sys/bsdioctl.h>
+#endif
+
+#ifdef AMIGA
+#include <arpa/inet.h>
+#include <dirent.h>
+#include <string.h>
+#include <netinet/tcp.h>
+#include <sys/acct.h>
+#include <sys/fcntl.h>
+#include <sys/filio.h>
+#include <sys/sockio.h>
+#include <netinet/in_systm.h>
+#include <netinet/ip.h>
+#include <sys/termios.h>
+#include <limits.h>
+#include <sys/timeb.h>
+
+#define SIGNAL_CAST (void (*)(int))
+#define USE_GETCWD
+#define HAVE_BZERO
+#define HAVE_MEMMOVE
+#define USE_SIGPROCMASK
+#define USE_WAITPID
+#define USE_DIRECT
+#define USE_F_FSIZE
+#define HAVE_FCNTL_LOCK 0
+#define HAVE_GETTIMEOFDAY
+#define HAVE_PATHCONF
+
+#define HAVE_NO_PROC
+#define NO_FORK_DEBUG
+#define HAVE_FORK 0
+#define HAVE_VFORK 1
+#endif
+
 
 /*******************************************************************
 end of the platform specific sections
 ********************************************************************/
 
+#if defined(USE_MMAP) || FAST_SHARE_MODES
+#include <sys/mman.h>
+#endif
+
+#ifdef SecureWare
+#define NEED_AUTH_PARAMETERS
+#endif
+
 #ifdef REPLACE_GETPASS
 extern char    *getsmbpass(char *);
 #define getpass(s) getsmbpass(s)
 #endif
 
-#ifndef FD_SETSIZE
-#define FD_SETSIZE 255
+#ifdef REPLACE_INNETGR
+#define innetgr(group,host,user,dom) InNetGr(group,host,user,dom)
 #endif
 
-#ifndef MAXINT
-#define MAXINT ((((unsigned)1)<<(sizeof(int)*8-1))-1)
+#ifndef FD_SETSIZE
+#define FD_SETSIZE 255
 #endif
 
 #ifndef __STDC__
@@ -852,10 +1071,9 @@ extern char *sys_errlist[];
 
 #include "version.h"
 #include "smb.h"
+#include "nameserv.h"
+#include "proto.h"
 #include "byteorder.h"
-#ifdef SMB_PASSWD
-#include "smbpass.h"
-#endif
 
 #include "kanji.h"
 #include "charset.h"
@@ -923,6 +1141,10 @@ it works and getting lots of bug reports */
 #define SIGCLD SIGCHLD
 #endif 
 
+#ifndef MAP_FILE
+#define MAP_FILE 0
+#endif
+
 #ifndef HAVE_FCNTL_LOCK
 #define HAVE_FCNTL_LOCK 1
 #endif
@@ -1025,3 +1247,5 @@ void free_wrapped(void *,char *,int);
 #define memcpy(d,s,l) memcpy_wrapped(d,s,l,__FILE__,__LINE__)
 void *memcpy_wrapped(void *d,void *s,int l,char *fname,int line);
 #endif
+
+#endif
