@@ -24,8 +24,6 @@
    */
 
 #include "includes.h"
-#include "loadparm.h"
-#include "pcap.h"
 
 #ifdef CHECK_TYPES
 #undef CHECK_TYPES
@@ -35,6 +33,7 @@
 extern int DEBUGLEVEL;
 extern int maxxmit;
 extern files_struct Files[];
+extern connection_struct Connections[];
 
 extern fstring local_machine;
 
@@ -55,18 +54,26 @@ extern fstring local_machine;
 #define ACCESS_WRITE 0x02
 #define ACCESS_CREATE 0x04
 
-#define STYPE_DISKTREE	0	/* Disk drive */
-#define STYPE_PRINTQ	1	/* Spooler queue */
-#define STYPE_DEVICE	2	/* Serial device */
-#define STYPE_IPC	3	/* Interprocess communication (IPC) */
-
 #define SHPWLEN 8		/* share password length */
 #define NNLEN 12		/* 8.3 net name length */
 #define SNLEN 15		/* service name length */
 #define QNLEN 12		/* queue name maximum length */
 
+#define MAJOR_VERSION 4
+#define MINOR_VERSION 1
+
 extern int Client;
 
+static BOOL api_Unsupported(int cnum,int uid, char *param,char *data,
+			    int mdrcnt,int mprcnt,
+			    char **rdata,char **rparam,
+			    int *rdata_len,int *rparam_len);
+static BOOL api_TooSmall(int cnum,int uid, char *param,char *data,
+			 int mdrcnt,int mprcnt,
+			 char **rdata,char **rparam,
+			 int *rdata_len,int *rparam_len);
+
+
 static int CopyExpanded(int cnum, int snum, char** dst, char* src, int* n)
 {
   pstring buf;
@@ -193,12 +200,6 @@ static void send_trans_reply(char *outbuf,char *data,char *param,uint16 *setup,
     }
 }
 
-
-
-/****************************************************************************
-  get a print queue
-  ****************************************************************************/
-
 struct pack_desc {
   char* format;	    /* formatstring for structure */
   char* subformat;  /* subformat for structure */
@@ -423,6 +424,10 @@ static void PACKS(struct pack_desc* desc,char *t,char *v)
 }
 
 
+/****************************************************************************
+  get a print queue
+  ****************************************************************************/
+
 static void PackDriverData(struct pack_desc* desc)
 {
   char drivdata[4+4+32];
@@ -472,7 +477,7 @@ static void fill_printjob_info(int cnum, int snum, int uLevel,
   time_t t = queue->time;
 
   /* the client expects localtime */
-  t += GMT_TO_LOCAL*TimeDiff(t);
+  t -= TimeDiff(t);
 
   PACKI(desc,"W",((snum%0xFF)<<8) | (queue->job%0xFF)); /* uJobId */
   if (uLevel == 1) {
@@ -484,7 +489,7 @@ static void fill_printjob_info(int cnum, int snum, int uLevel,
     PACKI(desc,"W",n+1);		/* uPosition */
     PACKI(desc,"W",queue->status); /* fsStatus */
     PACKS(desc,"z","");		/* pszStatus */
-    PACKI(desc,"D",queue->time); /* ulSubmitted */
+    PACKI(desc,"D",t); /* ulSubmitted */
     PACKI(desc,"D",queue->size); /* ulSize */
     PACKS(desc,"z",queue->file); /* pszComment */
   }
@@ -493,7 +498,7 @@ static void fill_printjob_info(int cnum, int snum, int uLevel,
     PACKS(desc,"z",queue->user); /* pszUserName */
     PACKI(desc,"W",n+1);		/* uPosition */
     PACKI(desc,"W",queue->status); /* fsStatus */
-    PACKI(desc,"D",queue->time); /* ulSubmitted */
+    PACKI(desc,"D",t); /* ulSubmitted */
     PACKI(desc,"D",queue->size); /* ulSize */
     PACKS(desc,"z","Samba");	/* pszComment */
     PACKS(desc,"z",queue->file); /* pszDocument */
@@ -550,7 +555,7 @@ static void fill_printq_info(int cnum, int snum, int uLevel,
     PACKI(desc,"W",0);		/* uUntiltime */
     PACKI(desc,"W",5);		/* pad1 */
     PACKS(desc,"z","");		/* pszSepFile */
-    PACKS(desc,"z","lpd");	/* pszPrProc */
+    PACKS(desc,"z","WinPrint");	/* pszPrProc */
     PACKS(desc,"z","");		/* pszParms */
     if (!status || !status->message[0]) {
       PACKS(desc,"z",Expand(cnum,snum,lp_comment(snum))); /* pszComment */
@@ -561,7 +566,7 @@ static void fill_printq_info(int cnum, int snum, int uLevel,
     }
     PACKI(desc,(uLevel == 3 ? "W" : "N"),count);	/* cJobs */
     PACKS(desc,"z",SERVICE(snum)); /* pszPrinters */
-    PACKS(desc,"z","NULL");	/* pszDriverName */
+    PACKS(desc,"z",lp_printerdriver(snum));		/* pszDriverName */
     PackDriverData(desc);	/* pDriverData */
   }
   if (uLevel == 2 || uLevel == 4) {
@@ -598,6 +603,7 @@ static BOOL api_DosPrintQGetInfo(int cnum,int uid, char *param,char *data,
   cbBuf = SVAL(p,2);
   str3 = p + 4;
  
+  /* remove any trailing username */
   if ((p = strchr(QueueName,'%'))) *p = 0;
  
   DEBUG(3,("PrintQueue uLevel=%d name=%s\n",uLevel,QueueName));
@@ -667,12 +673,12 @@ static BOOL api_DosPrintQEnum(int cnum, int uid, char* param, char* data,
 
   DEBUG(3,("DosPrintQEnum uLevel=%d\n",uLevel));
  
-  if (prefix_ok(param_format,"WrLeh")) return False;
+  if (!prefix_ok(param_format,"WrLeh")) return False;
   if (!check_printq_info(&desc,uLevel,output_format1,output_format2))
     return False;
   queuecnt = 0;
   for (i = 0; i < services; i++)
-    if (lp_print_ok(i) && lp_browseable(i))
+    if (lp_snum_ok(i) && lp_print_ok(i) && lp_browseable(i))
       queuecnt++;
   if (uLevel > 0) {
     queue = (print_queue_struct**)malloc(queuecnt*sizeof(print_queue_struct*));
@@ -683,7 +689,7 @@ static BOOL api_DosPrintQEnum(int cnum, int uid, char* param, char* data,
     subcnt = 0;
     n = 0;
     for (i = 0; i < services; i++)
-      if (lp_print_ok(i) && lp_browseable(i)) {
+      if (lp_snum_ok(i) && lp_print_ok(i) && lp_browseable(i)) {
  	subcntarr[n] = get_printqueue(i,cnum,&queue[n],&status[n]);
  	subcnt += subcntarr[n];
  	n++;
@@ -697,7 +703,7 @@ static BOOL api_DosPrintQEnum(int cnum, int uid, char* param, char* data,
     n = 0;
     succnt = 0;
     for (i = 0; i < services; i++)
-      if (lp_print_ok(i) && lp_browseable(i)) {
+      if (lp_snum_ok(i) && lp_print_ok(i) && lp_browseable(i)) {
 	fill_printq_info(cnum,i,uLevel,&desc,subcntarr[n],queue[n],&status[n]);
 	n++;
 	if (desc.errcode == NERR_Success) succnt = n;
@@ -742,10 +748,13 @@ static BOOL check_server_info(int uLevel, char* id)
   return True;
 }
 
-struct server_info_struct {
+struct srv_info_struct
+{
   fstring name;
   uint32 type;
   fstring comment;
+  fstring domain;
+  BOOL server_added;
 };
 
 
@@ -753,8 +762,9 @@ struct server_info_struct {
   get server info lists from the files saved by nmbd. Return the
   number of entries
   ******************************************************************/
-static int get_server_info(uint32 servertype,
-			   struct server_info_struct **servers)
+static int get_server_info(uint32 servertype, 
+			   struct srv_info_struct **servers,
+			   char *domain)
 {
   FILE *f;
   pstring fname;
@@ -774,50 +784,88 @@ static int get_server_info(uint32 servertype,
     return(0);
   }
 
-  if (servertype == SV_TYPE_ALL)
-    servertype &= ~SV_TYPE_DOMAIN_ENUM;
+  /* request for everything is code for request all servers */
+  if (servertype == SV_TYPE_ALL) servertype &= ~SV_TYPE_DOMAIN_ENUM;
 
-  while (!feof(f)) {
+  DEBUG(4,("Servertype search: %8x\n",servertype));
+
+  while (!feof(f))
+  {
     fstring stype;
-    struct server_info_struct *s;
+    struct srv_info_struct *s;
     char *ptr = line;
+    BOOL ok = True;
     *ptr = 0;
+
     fgets(line,sizeof(line)-1,f);
     if (!*line) continue;
-
+    
     if (count == alloced) {
       alloced += 10;
-      (*servers) = (struct server_info_struct *)
+      (*servers) = (struct srv_info_struct *)
 	Realloc(*servers,sizeof(**servers)*alloced);
       if (!(*servers)) return(0);
       bzero((char *)((*servers)+count),sizeof(**servers)*(alloced-count));
     }
     s = &(*servers)[count];
-
-    if (!next_token(&ptr,s->name,NULL)) continue;
-    if (!next_token(&ptr,stype,NULL)) continue;
-    if (!next_token(&ptr,s->comment,NULL)) continue;
-    if (sscanf(stype,"%X",&s->type) != 1) continue;
-
-    if (!s->type) s->type = SV_TYPE_SERVER;
-
-    if (!(servertype & s->type) || 
-	((s->type&SV_TYPE_DOMAIN_ENUM) && 
-	 !(servertype&SV_TYPE_DOMAIN_ENUM))) continue;
-
-    count++;
+    
+    if (!next_token(&ptr,s->name   , NULL)) continue;
+    if (!next_token(&ptr,stype     , NULL)) continue;
+    if (!next_token(&ptr,s->comment, NULL)) continue;
+    if (!next_token(&ptr,s->domain , NULL)) {
+      /* this allows us to cope with an old nmbd */
+      strcpy(s->domain,lp_workgroup()); 
+    }
+    
+    if (sscanf(stype,"%X",&s->type) != 1) { 
+      DEBUG(4,("r:host file ")); 
+      ok = False; 
+    }
+    
+    /* doesn't match up: don't want it */
+    if (!(servertype & s->type)) { 
+      DEBUG(4,("r:serv type ")); 
+      ok = False; 
+    }
+    
+    if ((servertype & SV_TYPE_DOMAIN_ENUM) != 
+	(s->type & SV_TYPE_DOMAIN_ENUM))
+      {
+	DEBUG(4,("s: dom mismatch "));
+	ok = False;
+      }
+    
+    if (!strequal(domain, s->domain) && !(servertype & SV_TYPE_DOMAIN_ENUM))
+      {
+	ok = False;
+      }
+    
+    if (ok)
+      {
+    	DEBUG(4,("**SV** %20s %8x %25s %15s\n",
+		 s->name, s->type, s->comment, s->domain));
+	
+    	s->server_added = True;
+    	count++;
+      }
+    else
+      {
+	DEBUG(4,("%20s %8x %25s %15s\n",
+		 s->name, s->type, s->comment, s->domain));
+      }
   }
-
+  
   fclose(f);
   return(count);
 }
 
+
 /*******************************************************************
   fill in a server info structure
   ******************************************************************/
-static int fill_server_info(struct server_info_struct *service,
-			    int uLevel,char **buf, int *buflen,
-			    char **stringbuf, int *stringspace, char *baseaddr)
+static int fill_srv_info(struct srv_info_struct *service, 
+			 int uLevel, char **buf, int *buflen, 
+			 char **stringbuf, int *stringspace, char *baseaddr)
 {
   int struct_len;
   char* p;
@@ -891,14 +939,18 @@ static int fill_server_info(struct server_info_struct *service,
 }
 
 
+static BOOL srv_comp(struct srv_info_struct *s1,struct srv_info_struct *s2)
+{
+  return(strcmp(s1->name,s2->name));
+}
+
 /****************************************************************************
   view list of servers available (or possibly domains). The info is
   extracted from lists saved by nmbd on the local host
   ****************************************************************************/
-static BOOL api_RNetServerEnum(int cnum,int uid, char *param,char *data,
-			       int mdrcnt,int mprcnt,
-			       char **rdata,char **rparam,
-			       int *rdata_len,int *rparam_len)
+static BOOL api_RNetServerEnum(int cnum, int uid, char *param, char *data,
+			       int mdrcnt, int mprcnt, char **rdata, 
+			       char **rparam, int *rdata_len, int *rparam_len)
 {
   char *str1 = param+2;
   char *str2 = skip_string(str1,1);
@@ -909,23 +961,59 @@ static BOOL api_RNetServerEnum(int cnum,int uid, char *param,char *data,
   char *p2;
   int data_len, fixed_len, string_len;
   int f_len, s_len;
-  struct server_info_struct *servers=NULL;
+  struct srv_info_struct *servers=NULL;
   int counted=0,total=0;
-  int i;
+  int i,missed;
+  fstring domain;
+  BOOL domain_request;
+  BOOL local_request = servertype & SV_TYPE_LOCAL_LIST_ONLY;
+
+  if (servertype == SV_TYPE_ALL) servertype &= ~SV_TYPE_DOMAIN_ENUM;
+
+  domain_request = ((servertype & SV_TYPE_DOMAIN_ENUM) != 0);
+
+  p += 8;
 
   if (!prefix_ok(str1,"WrLehD")) return False;
   if (!check_server_info(uLevel,str2)) return False;
   
-  total = get_server_info(servertype,&servers);
+  DEBUG(4, ("server request level: %s %8x ", str2, servertype));
+  DEBUG(4, ("domains_req:%s ", BOOLSTR(domain_request)));
+  DEBUG(4, ("local_only:%s\n", BOOLSTR(local_request)));
+
+  if (strcmp(str1, "WrLehDz") == 0) {
+    StrnCpy(domain, p, sizeof(fstring)-1);
+  } else {
+    StrnCpy(domain, lp_workgroup(), sizeof(fstring)-1);    
+  }
+
+  if (lp_browse_list())
+    total = get_server_info(servertype,&servers,domain);
 
   data_len = fixed_len = string_len = 0;
+  missed = 0;
 
-  for (i=0;i<total;i++) {
-    data_len += fill_server_info(&servers[i],uLevel,0,&f_len,0,&s_len,0);
-    if (data_len <= buf_len) {
-      counted++;
-      fixed_len += f_len;
-      string_len += s_len;
+  qsort(servers,total,sizeof(servers[0]),QSORT_CAST srv_comp);
+
+  {
+    char *lastname=NULL;
+
+    for (i=0;i<total;i++)
+    {
+      struct srv_info_struct *s = &servers[i];
+      if (lastname && strequal(lastname,s->name)) continue;
+      lastname = s->name;
+      data_len += fill_srv_info(s,uLevel,0,&f_len,0,&s_len,0);
+      DEBUG(4,("fill_srv_info %20s %8x %25s %15s\n",
+	       s->name, s->type, s->comment, s->domain));
+      
+      if (data_len <= buf_len) {
+	  counted++;
+	  fixed_len += f_len;
+	  string_len += s_len;
+      } else {
+	missed++;
+      }
     }
   }
 
@@ -933,29 +1021,42 @@ static BOOL api_RNetServerEnum(int cnum,int uid, char *param,char *data,
   *rdata = REALLOC(*rdata,*rdata_len);
   bzero(*rdata,*rdata_len);
   
-  p2 = (*rdata) + fixed_len;	/* auxillery data (strings) will go here */
+  p2 = (*rdata) + fixed_len;	/* auxilliary data (strings) will go here */
   p = *rdata;
   f_len = fixed_len;
   s_len = string_len;
-  for (i = 0; i < counted;i++)
-    fill_server_info(&servers[i],uLevel,&p,&f_len,&p2,&s_len,*rdata);
+
+  {
+    char *lastname=NULL;
+    int count2 = counted;
+    for (i = 0; i < total && count2;i++)
+      {
+	struct srv_info_struct *s = &servers[i];
+	if (lastname && strequal(lastname,s->name)) continue;
+	lastname = s->name;
+	fill_srv_info(s,uLevel,&p,&f_len,&p2,&s_len,*rdata);
+	DEBUG(4,("fill_srv_info %20s %8x %25s %15s\n",
+		 s->name, s->type, s->comment, s->domain));
+	count2--;
+      }
+  }
   
   *rparam_len = 8;
   *rparam = REALLOC(*rparam,*rparam_len);
   SSVAL(*rparam,0,NERR_Success);
   SSVAL(*rparam,2,0);
   SSVAL(*rparam,4,counted);
-  SSVAL(*rparam,6,total);
+  SSVAL(*rparam,6,counted+missed);
 
   if (servers) free(servers);
 
-  DEBUG(4,("NetServerEnum uLevel=%d counted=%d total=%d\n",uLevel,counted,total));
+  DEBUG(3,("NetServerEnum domain = %s uLevel=%d counted=%d total=%d\n",
+	   domain,uLevel,counted,counted+missed));
 
   return(True);
 }
 
 
-
 /****************************************************************************
   get info about a share
   ****************************************************************************/
@@ -1131,7 +1232,7 @@ static BOOL api_RNetShareEnum(int cnum,int uid, char *param,char *data,
   
   data_len = fixed_len = string_len = 0;
   for (i=0;i<count;i++)
-    if (lp_browseable(i))
+    if (lp_browseable(i) && lp_snum_ok(i))
       {
   	total++;
  	data_len += fill_share_info(cnum,i,uLevel,0,&f_len,0,&s_len,0);
@@ -1151,7 +1252,7 @@ static BOOL api_RNetShareEnum(int cnum,int uid, char *param,char *data,
   f_len = fixed_len;
   s_len = string_len;
   for (i = 0; i < count;i++)
-    if (lp_browseable(i))
+    if (lp_browseable(i) && lp_snum_ok(i))
       if (fill_share_info(cnum,i,uLevel,&p,&f_len,&p2,&s_len,*rdata) < 0)
  	break;
   
@@ -1200,7 +1301,7 @@ static BOOL api_NetRemoteTOD(int cnum,int uid, char *param,char *data,
 
     /* the client expects to get localtime, not GMT, in this bit 
        (I think, this needs testing) */
-    t = LocalTime(&unixdate,GMT_TO_LOCAL);
+    t = LocalTime(&unixdate);
 
     SIVAL(p,4,0);		/* msecs ? */
     CVAL(p,8) = t->tm_hour;
@@ -1243,14 +1344,16 @@ static BOOL api_SetUserPassword(int cnum,int uid, char *param,char *data,
 
   *rdata_len = 0;
 
-  SSVAL(*rparam,0,NERR_Success);
+  SSVAL(*rparam,0,NERR_badpass);
   SSVAL(*rparam,2,0);		/* converter word */
 
   DEBUG(3,("Set password for <%s>\n",user));
 
-  if (!password_ok(user,pass1,strlen(pass1),NULL,False) || 
-      !chgpasswd(user,pass1,pass2))
-    SSVAL(*rparam,0,NERR_badpass);
+  if (password_ok(user,pass1,strlen(pass1),NULL,False) &&
+      chgpasswd(user,pass1,pass2))
+  {
+    SSVAL(*rparam,0,NERR_Success);
+  }
 
   bzero(pass1,sizeof(fstring));
   bzero(pass2,sizeof(fstring));	 
@@ -1462,8 +1565,7 @@ static BOOL api_PrintJobInfo(int cnum,int uid,char *param,char *data,
 	int l = 0;
 	while (l<64 && *s)
 	  {
-	    if (isalnum(*s) || strchr("-._",*s))
-	      name[l++] = *s;
+	    if (issafe(*s)) name[l++] = *s;
 	    s++;
 	  }      
 	name[l] = 0;
@@ -1561,15 +1663,14 @@ static BOOL api_RNetServerGetInfo(int cnum,int uid, char *param,char *data,
   p += 16;
   if (uLevel > 0)
     {
-      struct server_info_struct *servers=NULL;
+      struct srv_info_struct *servers=NULL;
       int i,count;
       pstring comment;
-      uint32 servertype=SV_TYPE_SERVER_UNIX|SV_TYPE_WORKSTATION|
-	SV_TYPE_SERVER|SV_TYPE_TIME_SOURCE;
+      uint32 servertype=DFLT_SERVER_TYPE;
 
       strcpy(comment,lp_serverstring());
 
-      if ((count=get_server_info(SV_TYPE_ALL,&servers))>0) {
+      if ((count=get_server_info(SV_TYPE_ALL,&servers,lp_workgroup()))>0) {
 	for (i=0;i<count;i++)
 	  if (strequal(servers[i].name,local_machine)) {
 	    servertype = servers[i].type;
@@ -1578,9 +1679,10 @@ static BOOL api_RNetServerGetInfo(int cnum,int uid, char *param,char *data,
       }
       if (servers) free(servers);
 
-      SCVAL(p,0,2);		/* version_major */
-      SCVAL(p,1,0);		/* version_minor */
+      SCVAL(p,0,MAJOR_VERSION);
+      SCVAL(p,1,MINOR_VERSION);
       SIVAL(p,2,servertype);
+
       if (mdrcnt == struct_len) {
 	SIVAL(p,6,0);
       } else {
@@ -1651,16 +1753,16 @@ static BOOL api_NetWkstaGetInfo(int cnum,int uid, char *param,char *data,
   p += 4;
 
   SIVAL(p,0,PTR_DIFF(p2,*rdata));
-  strcpy(p2,my_workgroup());
+  strcpy(p2,lp_workgroup());
   p2 = skip_string(p2,1);
   p += 4;
 
-  SCVAL(p,0,2); /* major version?? */
-  SCVAL(p,1,1); /* minor version?? */
+  SCVAL(p,0,MAJOR_VERSION); 
+  SCVAL(p,1,MINOR_VERSION); 
   p += 2;
 
   SIVAL(p,0,PTR_DIFF(p2,*rdata));
-  strcpy(p2,my_workgroup());	/* login domain?? */
+  strcpy(p2,lp_workgroup());	/* login domain?? */
   p2 = skip_string(p2,1);
   p += 4;
 
@@ -1727,44 +1829,58 @@ static BOOL api_RNetUserGetInfo(int cnum,int uid, char *param,char *data,
     SCVAL(p,21,0);
     *p2 = 0;
     if (uLevel >= 10) {
-      SIVAL(p,22,PTR_DIFF(p2,*rdata)); /* comment */
-      SIVAL(p,26,PTR_DIFF(p2,*rdata)); /* user_comment */
-      SIVAL(p,30,PTR_DIFF(p2,*rdata)); /* full name */
-      p2++;
-    }
-    if (uLevel == 11) {
-      SSVAL(p,34,USER_PRIV_ADMIN); /* user privilege */
-      SIVAL(p,36,0);		/* auth flags */
-      SIVAL(p,40,0);		/* password age */
-      SIVAL(p,44,PTR_DIFF(p2,*rdata)); /* home dir */
-      strcpy(p2,"Y:\\GIGA\\C$\\PRJ\\USERS\\AD");
+      SIVAL(p,22,PTR_DIFF(p2,p)); /* comment */
+      strcpy(p2,"<Comment>");
       p2 = skip_string(p2,1);
-      SIVAL(p,48,PTR_DIFF(p2,*rdata)); /* parms */
+      SIVAL(p,26,PTR_DIFF(p2,p)); /* user_comment */
+      strcpy(p2,"<UserComment>");
+      p2 = skip_string(p2,1);
+      SIVAL(p,30,PTR_DIFF(p2,p)); /* full name */
+      strcpy(p2,"<FullName>");
+      p2 = skip_string(p2,1);
+    }
+    if (uLevel == 11) {         /* modelled after NTAS 3.51 reply */
+      SSVAL(p,34,
+	    Connections[cnum].admin_user?USER_PRIV_ADMIN:USER_PRIV_USER); 
+      SIVAL(p,36,0);		/* auth flags */
+      SIVALS(p,40,-1);		/* password age */
+      SIVAL(p,44,PTR_DIFF(p2,p)); /* home dir */
+      strcpy(p2,"\\\\%L\\HOMES");
+      standard_sub_basic(p2);
+      p2 = skip_string(p2,1);
+      SIVAL(p,48,PTR_DIFF(p2,p)); /* parms */
       strcpy(p2,"");
       p2 = skip_string(p2,1);
       SIVAL(p,52,0);		/* last logon */
       SIVAL(p,56,0);		/* last logoff */
       SSVALS(p,60,-1);		/* bad pw counts */
       SSVALS(p,62,-1);		/* num logons */
-      SIVAL(p,64,PTR_DIFF(p2,*rdata)); /* logon server */
-      strcpy(p2,"\\\\GIGA");
+      SIVAL(p,64,PTR_DIFF(p2,p)); /* logon server */
+      strcpy(p2,"\\\\*");
+      p2 = skip_string(p2,1);
+      SSVAL(p,68,0);		/* country code */
+
+      SIVAL(p,70,PTR_DIFF(p2,p)); /* workstations */
+      strcpy(p2,"");
       p2 = skip_string(p2,1);
-      SSVAL(p,68,49);		/* country code */
-      SIVAL(p,70,0);		/* workstations */
 
       SIVALS(p,74,-1);		/* max storage */
       SSVAL(p,78,168);		/* units per week */
-      SIVAL(p,80,PTR_DIFF(p2,*rdata)); /* logon hours */
+      SIVAL(p,80,PTR_DIFF(p2,p)); /* logon hours */
       memset(p2,-1,21);
-      p2 += 21;
-      SSVAL(p,84,860);		/* code page */
+      SCVAL(p2,21,0);           /* fix zero termination */
+      p2 = skip_string(p2,1);
+
+      SSVAL(p,84,0);		/* code page */
     }
     if (uLevel == 1 || uLevel == 2) {
       memset(p+22,' ',16);	/* password */
       SIVALS(p,38,-1);		/* password age */
-      SSVAL(p,42,USER_PRIV_ADMIN); /* user privilege */
+      SSVAL(p,42,
+	    Connections[cnum].admin_user?USER_PRIV_ADMIN:USER_PRIV_USER);
       SIVAL(p,44,PTR_DIFF(p2,*rdata)); /* home dir */
-      strcpy(p2,"\\\\__SAMBA__\\HOMES");
+      strcpy(p2,"\\\\%L\\HOMES");
+      standard_sub_basic(p2);
       p2 = skip_string(p2,1);
       SIVAL(p,48,PTR_DIFF(p2,*rdata)); /* comment */
       *p2++ = 0;
@@ -1791,7 +1907,8 @@ static BOOL api_RNetUserGetInfo(int cnum,int uid, char *param,char *data,
 	SSVALS(p,102,-1);	/* bad_pw_count */
 	SSVALS(p,104,-1);	/* num_logons */
 	SIVAL(p,106,PTR_DIFF(p2,*rdata)); /* logon_server */
-	strcpy(p2,"\\\\__SAMBA__");
+	strcpy(p2,"\\\\%L");
+	standard_sub_basic(p2);
 	p2 = skip_string(p2,1);
 	SSVAL(p,110,49);	/* country_code */
 	SSVAL(p,112,860);	/* code page */
@@ -1865,13 +1982,12 @@ static BOOL api_WWkstaUserLogon(int cnum,int uid, char *param,char *data,
   char *str1 = param+2;
   char *str2 = skip_string(str1,1);
   char *p = skip_string(str2,1);
-  int uLevel,cbBuf;
+  int uLevel;
   struct pack_desc desc;
   char* name;
 
   uLevel = SVAL(p,0);
-  cbBuf = SVAL(p,2);
-  name = p + 4;
+  name = p + 2;
 
   bzero(&desc,sizeof(desc));
 
@@ -1885,25 +2001,34 @@ static BOOL api_WWkstaUserLogon(int cnum,int uid, char *param,char *data,
   desc.buflen = mdrcnt;
   desc.subformat = NULL;
   desc.format = str2;
+  
+  
 
   if (init_package(&desc,1,0)) {
     PACKI(&desc,"W",0);		/* code */
     PACKS(&desc,"B21",name);	/* eff. name */
     PACKS(&desc,"B","");		/* pad */
-    PACKI(&desc,"W",2);		/* priv */
-    PACKI(&desc,"D",0);		/* auth flags */
+    PACKI(&desc,"W",
+	  Connections[cnum].admin_user?USER_PRIV_ADMIN:USER_PRIV_USER);
+    PACKI(&desc,"D",0);		/* auth flags XXX */
     PACKI(&desc,"W",0);		/* num logons */
     PACKI(&desc,"W",0);		/* bad pw count */
-    PACKI(&desc,"D",0);		/* last logon */
-    PACKI(&desc,"D",0);		/* last logoff */
+    PACKI(&desc,"D",-1);		/* last logon */
+    PACKI(&desc,"D",-1);		/* last logoff */
     PACKI(&desc,"D",-1);		/* logoff time */
     PACKI(&desc,"D",-1);		/* kickoff time */
     PACKI(&desc,"D",0);		/* password age */
     PACKI(&desc,"D",0);		/* password can change */
     PACKI(&desc,"D",-1);		/* password must change */
-    PACKS(&desc,"z",local_machine);/* computer */
-    PACKS(&desc,"z",my_workgroup());/* domain */
-    PACKS(&desc,"z","");		/* script path */
+    {
+      fstring mypath;
+      strcpy(mypath,"\\\\");
+      strcat(mypath,local_machine);
+      strupper(mypath);
+      PACKS(&desc,"z",mypath); /* computer */
+    }
+    PACKS(&desc,"z",lp_workgroup());/* domain */
+    PACKS(&desc,"z",lp_logon_script());		/* script path */
     PACKI(&desc,"D",0);		/* reserved */
   }
 
@@ -2219,7 +2344,7 @@ static BOOL api_WPrintDestEnum(int cnum,int uid, char *param,char *data,
 
   queuecnt = 0;
   for (i = 0; i < services; i++)
-    if (lp_print_ok(i) && lp_browseable(i))
+    if (lp_snum_ok(i) && lp_print_ok(i) && lp_browseable(i))
       queuecnt++;
 
   if (mdrcnt > 0) *rdata = REALLOC(*rdata,mdrcnt);
@@ -2229,7 +2354,7 @@ static BOOL api_WPrintDestEnum(int cnum,int uid, char *param,char *data,
     succnt = 0;
     n = 0;
     for (i = 0; i < services; i++) {
-      if (lp_print_ok(i) && lp_browseable(i)) {
+      if (lp_snum_ok(i) && lp_print_ok(i) && lp_browseable(i)) {
 	fill_printdest_info(cnum,i,uLevel,&desc);
 	n++;
 	if (desc.errcode == NERR_Success) succnt = n;
@@ -2388,6 +2513,92 @@ static BOOL api_WPrintPortEnum(int cnum,int uid, char *param,char *data,
   return(True);
 }
 
+
+struct
+{
+  char * name;
+  char * pipename;
+  int subcommand;
+  BOOL (*fn) ();
+} api_fd_commands [] =
+  {
+    { "SetNmdPpHndState",	"lsarpc",	1,	api_LsarpcSNPHS },
+    { "TransactNmPipe",	"lsarpc",	0x26,	api_LsarpcTNP },
+    { NULL,		NULL,		-1,	api_Unsupported }
+  };
+
+/****************************************************************************
+  handle remote api calls delivered to a named pipe already opened.
+  ****************************************************************************/
+static int api_fd_reply(int cnum,int uid,char *outbuf,
+		 	uint16 *setup,char *data,char *params,
+		 	int suwcnt,int tdscnt,int tpscnt,int mdrcnt,int mprcnt)
+{
+  char *rdata = NULL;
+  char *rparam = NULL;
+  int rdata_len = 0;
+  int rparam_len = 0;
+  BOOL reply=False;
+  int i;
+  int fd;
+  int subcommand;
+  
+  /* First find out the name of this file. */
+  if (suwcnt != 2)
+    {
+      DEBUG(0,("Unexpected named pipe transaction.\n"));
+      return(-1);
+    }
+  
+  /* Get the file handle and hence the file name. */
+  fd = setup[1];
+  subcommand = setup[0];
+  
+  DEBUG(3,("Got API command %d on pipe %s ",subcommand,Files[fd].name));
+  DEBUG(3,("(tdscnt=%d,tpscnt=%d,mdrcnt=%d,mprcnt=%d)\n",
+	   tdscnt,tpscnt,mdrcnt,mprcnt));
+  
+  for (i=0;api_fd_commands[i].name;i++)
+    if (strequal(api_fd_commands[i].pipename, Files[fd].name) &&
+	api_fd_commands[i].subcommand == subcommand &&
+	api_fd_commands[i].fn)
+      {
+	DEBUG(3,("Doing %s\n",api_fd_commands[i].name));
+	break;
+      }
+  
+  rdata = (char *)malloc(1024); if (rdata) bzero(rdata,1024);
+  rparam = (char *)malloc(1024); if (rparam) bzero(rparam,1024);
+  
+  reply = api_fd_commands[i].fn(cnum,uid,params,data,mdrcnt,mprcnt,
+			        &rdata,&rparam,&rdata_len,&rparam_len);
+  
+  if (rdata_len > mdrcnt ||
+      rparam_len > mprcnt)
+    {
+      reply = api_TooSmall(cnum,uid,params,data,mdrcnt,mprcnt,
+			   &rdata,&rparam,&rdata_len,&rparam_len);
+    }
+  
+  
+  /* if we get False back then it's actually unsupported */
+  if (!reply)
+    api_Unsupported(cnum,uid,params,data,mdrcnt,mprcnt,
+		    &rdata,&rparam,&rdata_len,&rparam_len);
+  
+  /* now send the reply */
+  send_trans_reply(outbuf,rdata,rparam,NULL,rdata_len,rparam_len,0);
+  
+  if (rdata)
+    free(rdata);
+  if (rparam)
+    free(rparam);
+  
+  return(-1);
+}
+
+
+
 /****************************************************************************
   the buffer was too small
   ****************************************************************************/
@@ -2538,6 +2749,10 @@ static int named_pipe(int cnum,int uid, char *outbuf,char *name,
   if (strequal(name,"LANMAN"))
     return(api_reply(cnum,uid,outbuf,data,params,tdscnt,tpscnt,mdrcnt,mprcnt));
 
+if (strlen(name) < 1)
+  return(api_fd_reply(cnum,uid,outbuf,setup,data,params,suwcnt,tdscnt,tpscnt,mdrcnt,mprcnt));
+
+
   DEBUG(3,("named pipe command on <%s> 0x%X setup1=%d\n",
 	   name,(int)setup[0],(int)setup[1]));
   
@@ -2607,12 +2822,9 @@ int reply_trans(char *inbuf,char *outbuf)
   while (pscnt < tpscnt || dscnt < tdscnt)
     {
       int pcnt,poff,dcnt,doff,pdisp,ddisp;
-
-      receive_smb(Client,inbuf, 0);
-      show_msg(inbuf);
-	  
-      /* Ensure this is still a trans packet (sanity check) */
-      if(CVAL(inbuf, smb_com) != SMBtrans)
+      
+      if (!receive_smb(Client,inbuf, SMB_SECONDARY_WAIT) ||
+	  CVAL(inbuf, smb_com) != SMBtrans)
 	{
 	  DEBUG(2,("Invalid secondary trans2 packet\n"));
 	  if (params) free(params);
@@ -2620,6 +2832,8 @@ int reply_trans(char *inbuf,char *outbuf)
 	  if (setup) free(setup);
 	  return(ERROR(ERRSRV,ERRerror));
 	}
+
+      show_msg(inbuf);
       
       tpscnt = SVAL(inbuf,smb_vwv0);
       tdscnt = SVAL(inbuf,smb_vwv1);
