@@ -53,17 +53,6 @@
 /* this is where browse lists are kept in the lock dir */
 #define SERVER_LIST "browse.dat"
 
-/* the print command on the server, %s is replaced with the filename  */
-/* note that the -r removes the file after printing - you'll run out  */
-/* of disk pretty quickly if you don't. This command is only used as  */
-/* the default - it can be overridden in the configuration file.      */
-#define PRINT_COMMAND "lpr -r %s"
-
-/* the lpq command on the server. the printername is passed as an argument */
-#ifndef LPQ_COMMAND
-#define LPQ_COMMAND "lpq -P"
-#endif
-
 /* shall guest entries in printer queues get changed to user entries,
    so they can be deleted using the windows print manager? */
 #define LPQ_GUEST_TO_USER
@@ -83,12 +72,6 @@
    manager window? */
 #define FSTYPE_STRING "Samba"
 
-/* we have two time standards - local and GMT. This will try to sort them out.
- */
-
-#define LOCAL_TO_GMT 1
-#define GMT_TO_LOCAL (-1)
-
 /* do you want smbd to send a 1 byte packet to nmbd to trigger it to start 
    when smbd starts? */
 #ifndef PRIME_NMBD
@@ -133,10 +116,10 @@
 #define SMBD_RELOAD_CHECK (10)
 #define SHARE_MODES_CHECK (10)
 #define SHARE_MODES_CLEAN (300)
-#define IDLE_CLOSED_TIMEOUT (30)
+#define IDLE_CLOSED_TIMEOUT (60)
 #define DPTR_IDLE_TIMEOUT (120)
 #define SMBD_SELECT_LOOP (10)
-#define NMBD_SELECT_LOOP (10)
+#define NMBD_SELECT_LOOP (2)
 #define BROWSE_INTERVAL (60)
 #define REGISTRATION_INTERVAL (10*60)
 #define NMBD_INETD_TIMEOUT (120)
@@ -163,5 +146,12 @@
 /* shall we support browse requests via a FIFO to nmbd? */
 #define ENABLE_FIFO 1
 
+/* keep the password server open, this uses up a aocket, but is needed
+   by many apps */
+#define KEEP_PASSWORD_SERVER_OPEN 1
+
+/* how long to wait for a socket connect to happen */
+#define LONG_CONNECT_TIMEOUT 30
+#define SHORT_CONNECT_TIMEOUT 5
 
 #endif
