@@ -20,7 +20,6 @@
 */
 
 #include "includes.h"
-#include "loadparm.h"
 
 extern int DEBUGLEVEL;
 extern int case_default;
@@ -82,14 +81,13 @@ return True if a name is in 8.3 dos format
 ****************************************************************************/
 BOOL is_8_3(char *fname)
 {
-  int len = strlen(fname);
+  int len;
   char *dot_pos;
   char *slash_pos = strrchr(fname,'/');
   int l;
 
   if (slash_pos) fname = slash_pos+1;
-
-  dot_pos = strchr(fname,'.');
+  len = strlen(fname);
 
   DEBUG(5,("checking %s for 8.3\n",fname));
 
@@ -121,6 +119,8 @@ BOOL is_8_3(char *fname)
   /* %%% A nice improvment to name mangling would be to translate
      filename to ANSI charset on the smb server host */
 
+  dot_pos = strchr(fname,'.');
+
   {
     char *p = fname;
 #ifdef KANJI
@@ -383,13 +383,10 @@ BOOL is_mangled(char *s)
 /****************************************************************************
 return a base 36 character. v must be from 0 to 35.
 ****************************************************************************/
-static char base36(int v)
+static char base36(unsigned int v)
 {
-  v = v % 36;
-  if (v < 10)
-    return('0'+v);
-  else /* needed to work around a DEC C compiler bug */
-    return('A' + (v-10));
+  static char basechars[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+  return basechars[v % 36];
 }
 
 
@@ -509,9 +506,36 @@ void mangle_name_83(char *s)
 	  *p++ = 0;
 	  while (*p && extlen < 3)
 	    {
+#ifdef KANJI
+	      if (is_shift_jis (*p))
+		{
+		  if (extlen < 2)
+		    {
+		      extension[extlen++] = p[0];
+		      extension[extlen++] = p[1];
+		    }
+		  else 
+		    {
+		      extension[extlen++] = base36 (((unsigned char) *p) % 36);
+		    }
+		  p += 2;
+		}
+	      else if (is_kana (*p))
+		{
+		  extension[extlen++] = p[0];
+		  p++;
+		}
+	      else 
+		{
+		  if (isdoschar (*p) && *p != '.')
+		    extension[extlen++] = p[0];
+		  p++;
+		}
+#else
 	      if (isdoschar(*p) && *p != '.')
 		extension[extlen++] = *p;
 	      p++;
+#endif /* KANJI */
 	    }
 	  extension[extlen] = 0;
 	}
@@ -521,9 +545,36 @@ void mangle_name_83(char *s)
 
   while (*p && baselen < 5)
     {
+#ifdef KANJI
+      if (is_shift_jis (*p))
+	{
+	  if (baselen < 4)
+	    {
+	      base[baselen++] = p[0];
+	      base[baselen++] = p[1];
+	    }
+	  else 
+	    {
+	      base[baselen++] = base36 (((unsigned char) *p) % 36);
+	    }
+	  p += 2;
+	}
+      else if (is_kana (*p))
+	{
+	  base[baselen++] = p[0];
+	  p++;
+	}
+      else 
+	{
+	  if (isdoschar (*p) && *p != '.')
+	    base[baselen++] = p[0];
+	  p++;
+	}
+#else
       if (isdoschar(*p) && *p != '.')
 	base[baselen++] = *p;
       p++;
+#endif /* KANJI */
     }
   base[baselen] = 0;
 
