@@ -20,19 +20,20 @@
    
 */
 
+#ifdef SYSLOG
+#undef SYSLOG
+#endif
+
 #include "includes.h"
-#include "nameserv.h"
 
 extern int DEBUGLEVEL;
 
 extern pstring scope;
 
-extern struct in_addr bcast_ip;
 extern pstring myhostname;
+extern struct in_addr ipzero;
 
-static BOOL got_bcast = False;
-
-int ServerFD=-1;
+int ServerFD= -1;
 
 /****************************************************************************
   open the socket communication
@@ -48,7 +49,7 @@ static BOOL open_sockets(void)
       return False;
     }   
 
-  ServerFD = open_socket_in(SOCK_DGRAM, 0,3);
+  ServerFD = open_socket_in(SOCK_DGRAM, 0,3,interpret_addr(lp_socket_address()));
 
   if (ServerFD == -1)
     return(False);
@@ -65,24 +66,9 @@ static BOOL open_sockets(void)
 ****************************************************************************/
 static BOOL init_structs(void )
 {
-  struct in_addr myip;
-
-  if (!get_myname(myhostname,&myip))
+  if (!get_myname(myhostname,NULL))
     return(False);
 
-  /* Read the broadcast address from the interface */
-  {
-    struct in_addr ip0,ip2;
-
-    ip0 = myip;
-
-    if (!got_bcast) {
-      get_broadcast(&ip0,&bcast_ip,&ip2);
-
-      DEBUG(2,("Using broadcast %s\n",inet_ntoa(bcast_ip)));
-    }
-  }
-
   return True;
 }
 
@@ -107,32 +93,33 @@ static void usage(void)
 int main(int argc,char *argv[])
 {
   int opt;
-  unsigned int lookup_type = 0x20;
+  unsigned int lookup_type = 0x0;
   pstring lookup;
-  extern FILE *dbf;
   extern int optind;
   extern char *optarg;
   BOOL find_master=False;
   BOOL find_status=False;
   int i;
+  static pstring servicesf = CONFIGFILE;
+  struct in_addr bcast_addr;
+  BOOL got_bcast = False;
   
   DEBUGLEVEL = 1;
   *lookup = 0;
 
   TimeInit();
+
+  setup_logging(argv[0],True);
+
   charset_initialise();
 
-  dbf = stdout;
-
-  while ((opt = getopt(argc, argv, "p:d:B:i:SMh")) != EOF)
+  while ((opt = getopt(argc, argv, "p:d:B:i:s:SMh")) != EOF)
     switch (opt)
       {
       case 'B':
-	{
-	  unsigned long a = interpret_addr(optarg);
-	  putip((char *)&bcast_ip,(char *)&a);
-	  got_bcast = True;
-	}
+	iface_set_default(NULL,optarg,NULL);
+	bcast_addr = *interpret_addr2(optarg);
+	got_bcast = True;
 	break;
       case 'i':
 	strcpy(scope,optarg);
@@ -147,6 +134,9 @@ int main(int argc,char *argv[])
       case 'd':
 	DEBUGLEVEL = atoi(optarg);
 	break;
+      case 's':
+	strcpy(servicesf, optarg);
+	break;
       case 'h':
 	usage();
 	exit(0);
@@ -161,14 +151,22 @@ int main(int argc,char *argv[])
     exit(1);
   }
 
+  if (!lp_load(servicesf,True)) {
+    fprintf(stderr, "Can't load %s - run testparm to debug it\n", servicesf);
+  }
+
+  load_interfaces();
   init_structs();
   if (!open_sockets()) return(1);
 
-  DEBUG(1,("Sending queries to %s\n",inet_ntoa(bcast_ip)));
+  if (!got_bcast)
+    bcast_addr = *iface_bcast(ipzero);
+
+  DEBUG(1,("Sending queries to %s\n",inet_ntoa(bcast_addr)));
 
 
   for (i=optind;i<argc;i++)
-    {
+  {
       BOOL bcast = True;
       int retries = 2;
       char *p;
@@ -194,20 +192,18 @@ int main(int argc,char *argv[])
 	retries = 1;
       }
 
-      if (name_query(ServerFD,lookup,lookup_type,bcast,True,
-		     bcast_ip,&ip,NULL)) 
+    if (name_query(ServerFD,lookup,lookup_type,bcast,True,
+		   bcast_addr,&ip,NULL)) 
 	{
 	  printf("%s %s\n",inet_ntoa(ip),lookup);
-	  if (find_status) 
-	    {
+    }
+	if (find_status) 
+	{
 	      printf("Looking up status of %s\n",inet_ntoa(ip));
 	      name_status(ServerFD,lookup,lookup_type,True,ip,NULL,NULL,NULL);
 	      printf("\n");
-	    }
-      } else {
-	printf("couldn't find name %s\n",lookup);
-      }
-    }
+	}
+  }
 
   return(0);
 }
