@@ -20,13 +20,17 @@
 */
 
 #include "includes.h"
-#include "loadparm.h"
 
 extern int DEBUGLEVEL;
 extern int Protocol;
 
 /* users from session setup */
-pstring session_users="";
+static pstring session_users="";
+
+/* these are kept here to keep the string_combinations function simple */
+static char this_user[100]="";
+static char this_salt[100]="";
+static char this_crypted[100]="";
 
 #ifdef SMB_PASSWD
 /* Data to do lanman1/2 password challenge. */
@@ -38,7 +42,6 @@ Get the next challenge value - no repeats.
 ********************************************************************/
 void generate_next_challenge(char *challenge)
 {
-  extern void E1(char *,char *,char *);
   static int counter = 0;
   struct timeval tval;
   int v1,v2;
@@ -47,11 +50,21 @@ void generate_next_challenge(char *challenge)
   v2 = (counter++) * getpid() + tval.tv_usec;
   SIVAL(challenge,0,v1);
   SIVAL(challenge,4,v2);
-  E1(challenge,"SAMBA",saved_challenge);
+  E1(challenge,"SAMBA",(char *)saved_challenge);
   memcpy(challenge,saved_challenge,8);
   challenge_sent = True;
 }
 
+/*******************************************************************
+set the last challenge sent, usually from a password server
+********************************************************************/
+BOOL set_challenge(char *challenge)
+{
+  memcpy(saved_challenge,challenge,8);
+  challenge_sent = True;
+  return(True);
+}
+
 /*******************************************************************
 get the last challenge sent
 ********************************************************************/
@@ -272,15 +285,13 @@ static void update_protected_database( char *user, BOOL result)
 #ifdef OSF1_ENH_SEC
   struct pr_passwd *mypasswd;
   time_t starttime;
-  long tz;
 
   mypasswd = getprpwnam (user);
   starttime = time (NULL);
-  tz = mktime ( localtime ( &starttime ) );
 
   if (result)
     {
-      mypasswd->ufld.fd_slogin = tz;
+      mypasswd->ufld.fd_slogin = starttime;
       mypasswd->ufld.fd_nlogins = 0;
       
       putprpwnam(user,mypasswd);
@@ -289,7 +300,7 @@ static void update_protected_database( char *user, BOOL result)
     }
   else
     {
-      mypasswd->ufld.fd_ulogin = tz;
+      mypasswd->ufld.fd_ulogin = starttime;
       mypasswd->ufld.fd_nlogins = mypasswd->ufld.fd_nlogins + 1;
       if ( mypasswd->ufld.fd_max_tries != 0 && mypasswd->ufld.fd_nlogins > mypasswd->ufld.fd_max_tries )
 	{
@@ -333,13 +344,15 @@ static BOOL afs_auth(char *this_user,char *password)
 
 
 #ifdef DFS_AUTH
+
+sec_login_handle_t my_dce_sec_context;
+int dcelogin_atmost_once = 0;
+
 /*******************************************************************
 check on a DCE/DFS authentication
 ********************************************************************/
 static BOOL dfs_auth(char *this_user,char *password)
 {
-  static sec_login_handle_t my_dce_sec_context;
-  static int dcelogin_atmost_once = 0;
   error_status_t err;
   int err2;
   int prterr;
@@ -394,7 +407,7 @@ static BOOL dfs_auth(char *this_user,char *password)
       dce_error_inq_text(err, dce_errstr, &err2);
       DEBUG(0,("DCE login failed for principal %s, cant set context: %s\n",
 	       this_user,dce_errstr));
-      sec_login_purge_context(my_dce_sec_context, &err2);
+      sec_login_purge_context(my_dce_sec_context, &err);
       return(False);
     }
   else
@@ -406,6 +419,22 @@ static BOOL dfs_auth(char *this_user,char *password)
   dcelogin_atmost_once = 1;
   return (True);
 }
+
+void dfs_unlogin(void)
+{
+  error_status_t err;
+  int err2;
+  unsigned char dce_errstr[dce_c_error_string_len];
+
+  sec_login_purge_context(my_dce_sec_context, &err);
+  if (err != error_status_ok )
+    {  
+      dce_error_inq_text(err, dce_errstr, &err2);
+      DEBUG(0,("DCE purge login context failed for server instance %d: %s\n",
+	       getpid(), dce_errstr));
+    }
+}
+
 #endif
 
 
@@ -482,11 +511,6 @@ static BOOL string_combinations(char *s,BOOL (*fn)(),int N)
 }
 
 
-/* these are kept here to keep the string_combinations function simple */
-char this_user[100]="";
-char this_salt[100]="";
-char this_crypted[100]="";
-
 
 /****************************************************************************
 core of password checking routine
@@ -518,7 +542,12 @@ BOOL password_check(char *password)
   return(linux_bigcrypt(password,this_salt,this_crypted));
 #endif
 
+#ifdef NO_CRYPT
+  DEBUG(1,("Warning - no crypt available\n"));
+  return(False);
+#else
   return(strcmp((char *)crypt(password,this_salt),this_crypted) == 0);
+#endif
 }
 
 #ifdef SMB_PASSWD
@@ -651,7 +680,9 @@ BOOL password_ok(char *user,char *password, int pwlen, struct passwd *pwd, BOOL
 		if(smb_pass->smb_nt_passwd != NULL)
 		{
 		  DEBUG(4,("Checking NT MD4 password\n"));
-	      if(smb_password_check(password, smb_pass->smb_nt_passwd, challenge))
+		  if(smb_password_check(password, 
+					smb_pass->smb_nt_passwd, 
+					(char *)challenge))
    		  {
 	      	update_protected_database(user,True);
 	        return(True);
@@ -663,11 +694,12 @@ BOOL password_ok(char *user,char *password, int pwlen, struct passwd *pwd, BOOL
 
 	/* Try against the lanman password */
 
-      if(smb_password_check(password, smb_pass->smb_passwd, challenge))
-	{
-	  update_protected_database(user,True);
-	  return(True);
-	}
+      if (smb_password_check(password, 
+			     smb_pass->smb_passwd,
+			     (char *)challenge)) {
+	update_protected_database(user,True);
+	return(True);
+      }
 
 	DEBUG(3,("Error smb_password_check failed\n"));
     }
@@ -683,7 +715,14 @@ BOOL password_ok(char *user,char *password, int pwlen, struct passwd *pwd, BOOL
 
 #ifdef SHADOW_PWD
   {
-    struct spwd *spass = getspnam(pass->pw_name);
+    struct spwd *spass;
+
+    /* many shadow systems require you to be root to get the password,
+       in most cases this should already be the case when this
+       function is called, except perhaps for IPC password changing
+       requests */
+
+    spass = getspnam(pass->pw_name);
     if (spass && spass->sp_pwdp)
       pass->pw_passwd = spass->sp_pwdp;
   }
@@ -697,6 +736,14 @@ BOOL password_ok(char *user,char *password, int pwlen, struct passwd *pwd, BOOL
   }
 #endif
 
+#ifdef HPUX_10_TRUSTED
+  {
+    struct pr_passwd *pr_pw = getprpwnam(pass->pw_name);
+    if (pr_pw && pr_pw->ufld.fd_encrypt)
+      pass->pw_passwd = pr_pw->ufld.fd_encrypt;
+  }
+#endif
+
 #ifdef OSF1_ENH_SEC
   {
     struct pr_passwd *mypasswd;
@@ -1105,13 +1152,12 @@ static BOOL check_user_equiv(char *user, char *remote, char *equiv_file)
 	  BOOL host_ok = False;
 
 #ifdef NETGROUP	  
-	  /* THIS IS UNTESTED!! */
 	  if (is_group)
 	    {
 	      static char *mydomain = NULL;
 	      if (!mydomain)
 		yp_get_default_domain(&mydomain);
-	      if (mydomain && innetgr(remote,file_host,user,mydomain))
+	      if (mydomain && innetgr(file_host,remote,user,mydomain))
 		host_ok = True;
 	    }
 #else
@@ -1194,7 +1240,8 @@ BOOL check_hosts_equiv(char *user)
 }
 
 
-static int password_client = -1;
+int password_client = -1;
+static fstring pserver;
 
 /****************************************************************************
 attempted support for server level security 
@@ -1208,15 +1255,12 @@ BOOL server_cryptkey(char *buf)
   int len;
   fstring desthost;
   struct in_addr dest_ip;
-  extern struct in_addr myip;
-  int port = 139;
-  char *pserver = lp_passwordserver();
+  int port = SMB_PORT;
   BOOL ret;
 
-  if (password_client >= 0) {
+  if (password_client >= 0)
     close(password_client);
-    password_client=-1;
-  }
+  password_client = -1;
 
   if (Protocol < PROTOCOL_NT1) {
     strcpy(pass_protocol,"LM1.2X002");
@@ -1227,28 +1271,35 @@ BOOL server_cryptkey(char *buf)
   bzero(inbuf,sizeof(inbuf));
   bzero(outbuf,sizeof(outbuf));
 
-  strcpy(desthost,pserver);
-  standard_sub_basic(desthost);
-  strupper(desthost);
+  for (p=strtok(lp_passwordserver(),LIST_SEP); p ; p = strtok(NULL,LIST_SEP)) {
+    strcpy(desthost,p);
+    standard_sub_basic(desthost);
+    strupper(desthost);
 
-  dest_ip = *interpret_addr2(desthost);
-  if (zero_ip(dest_ip)) {
-    DEBUG(1,("Can't find password server\n"));
-    return(False);
+    dest_ip = *interpret_addr2(desthost);
+    if (zero_ip(dest_ip)) {
+      DEBUG(1,("Can't resolve address for %s\n",p));
+      continue;
+    }
+
+    if (ismyip(dest_ip)) {
+      DEBUG(1,("Password server loop - disabling password server %s\n",p));
+      continue;
+    }
+
+    password_client = open_socket_out(SOCK_STREAM, &dest_ip, port, SHORT_CONNECT_TIMEOUT);
+    if (password_client >= 0) {
+      DEBUG(3,("connected to password server %s\n",p));
+      StrnCpy(pserver,p,sizeof(pserver)-1);
+      break;
+    }
   }
 
-  if (memcmp(&dest_ip,&myip,sizeof(dest_ip)) == 0) {
-    DEBUG(1,("Password server loop - disabling password server\n"));
-    return(False);
-  }
-
-  password_client = open_socket_out(SOCK_STREAM, &dest_ip, port);
   if (password_client < 0) {
-    DEBUG(1,("%s not available\n",pserver));
+    DEBUG(1,("password server not available\n"));
     return(False);
   }
 
-  DEBUG(3,("connected to password server %s\n",pserver));
 
   /* send a session request (RFC 8002) */
 
@@ -1267,9 +1318,10 @@ BOOL server_cryptkey(char *buf)
   CVAL(outbuf,0) = 0x81;
 
   send_smb(password_client,outbuf);
-  receive_smb(password_client,inbuf,5000);
+  
  
-  if (CVAL(inbuf,0) != 0x82) {
+  if (!receive_smb(password_client,inbuf,5000) ||
+      CVAL(inbuf,0) != 0x82) {
     DEBUG(1,("%s rejected the session\n",pserver));
     close(password_client); password_client = -1;
     return(False);
@@ -1294,13 +1346,13 @@ BOOL server_cryptkey(char *buf)
 
   if (!ret || CVAL(inbuf,smb_rcls) || SVAL(inbuf,smb_vwv0)) {
     DEBUG(1,("%s rejected the protocol\n",pserver));
-    close(password_client); password_client=-1;
+    close(password_client); password_client= -1;
     return(False);
   }
 
   if (!(CVAL(inbuf,smb_vwv1) & 1)) {
     DEBUG(1,("%s isn't in user level security mode\n",pserver));
-    close(password_client); password_client=-1;
+    close(password_client); password_client= -1;
     return(False);
   }
 
@@ -1317,7 +1369,6 @@ attempted support for server level security
 BOOL server_validate(char *buf)
 {
   pstring inbuf,outbuf;  
-  char *pserver = lp_passwordserver();
   BOOL ret;
 
   if (password_client < 0) {
@@ -1353,8 +1404,8 @@ BOOL server_validate(char *buf)
 
   DEBUG(3,("password server %s accepted the password\n",pserver));
 
-#ifndef KEEP_PASSWORD_SERVER_OPEN
-  close(password_client); password_client=-1;
+#if !KEEP_PASSWORD_SERVER_OPEN
+  close(password_client); password_client= -1;
 #endif
 
   return(True);
