@@ -54,8 +54,6 @@
 #include "includes.h"
 
 #include "smb.h"
-#include "loadparm.h"
-#include "pcap.h"
 
 extern int DEBUGLEVEL;
 
@@ -117,7 +115,7 @@ static void ScanQconfig_fn(char *psz,void (*fn)())
 			 if ((p=strchr(line,':')))
 			 {
 			 	*p = '\0';
-				p = strtok(line,':');
+				p = strtok(line,":");
 				if (strcmp(p,"bsh")!=0)
 				  {
 				    strcpy(name,p);
