@@ -20,7 +20,6 @@
 */
 
 #include "includes.h"
-#include "loadparm.h"
 extern int DEBUGLEVEL;
 extern connection_struct Connections[];
 extern files_struct Files[];
@@ -74,7 +73,7 @@ static char *build_print_command(int cnum, char *command, char *syscmd, char *fi
   }
   
   if (strstr(syscmd,"%s")) {
-    int iOffset = strstr(syscmd, "%s") - syscmd;
+    int iOffset = PTR_DIFF(strstr(syscmd, "%s"),syscmd);
     
     /* construct the full path for the filename, shouldn't be necessary unless
        the subshell causes a "cd" to be executed.
@@ -131,7 +130,7 @@ void print_file(int fnum)
   tempstr = build_print_command(cnum, PRINTCOMMAND(snum), syscmd, Files[fnum].name);
   if (tempstr != NULL)
     {
-      int ret = smbrun(syscmd,NULL);
+      int ret = smbrun(syscmd,NULL,False);
       DEBUG(3,("Running the command `%s' gave %d\n",syscmd,ret));
     }
   else
@@ -149,7 +148,7 @@ process time fields
 ********************************************************************/
 static time_t EntryTime(string tok[], int ptr, int count, int minimum)
 {
-  time_t jobtime;
+  time_t jobtime,jobtime1;
 
   jobtime = time(NULL);		/* default case: take current time */
   if (count >= minimum) {
@@ -181,7 +180,9 @@ static time_t EntryTime(string tok[], int ptr, int count, int minimum)
       t->tm_hour = hour;
       t->tm_min = min;
       t->tm_sec = sec;
-      jobtime = mktime(t);
+      jobtime1 = mktime(t);
+      if (jobtime1 != (time_t)-1)
+	jobtime = jobtime1;
     }
   }
   return jobtime;
@@ -276,6 +277,157 @@ static BOOL parse_lpq_bsd(char *line,print_queue_struct *buf,BOOL first)
   return(True);
 }
 
+/*
+<magnus@hum.auc.dk>
+LPRng_time modifies the current date by inserting the hour and minute from
+the lpq output.  The lpq time looks like "23:15:07"
+*/
+static time_t LPRng_time(string tok[],int pos)
+{
+  time_t jobtime;
+  struct tm *t;
+  char tmp_time[9];
+
+  jobtime = time(NULL);         /* default case: take current time */
+  t = localtime(&jobtime);
+  t->tm_hour = atoi(tok[pos]);
+  StrnCpy(tmp_time,tok[pos],sizeof(tmp_time));
+  t->tm_min = atoi(tmp_time+3);
+  t->tm_sec = atoi(tmp_time+6);
+  jobtime = mktime(t);
+
+  return jobtime;
+}
+
+
+/****************************************************************************
+  parse a lpq line
+  <magnus@hum.auc.dk>
+  Most of the code is directly reused from parse_lpq_bsd()
+
+here are two examples of lpq output under lprng (LPRng-2.3.0)
+
+Printer: humprn@hum-fak
+  Queue: 1 printable job
+  Server: pid 4840 active, Unspooler: pid 4841 active
+  Status: job 'cfA659hum-fak', closing device at Fri Jun 21 10:10:21 1996
+ Rank  Owner           Class Job Files                           Size Time
+active magnus@hum-fak      A 659 /var/spool/smb/Notesblok-ikke-na4024 10:03:31
+ 
+Printer: humprn@hum-fak (printing disabled)
+  Queue: 1 printable job
+  Warning: no server present
+  Status: finished operations at Fri Jun 21 10:10:32 1996
+ Rank  Owner           Class Job Files                           Size Time
+1      magnus@hum-fak      A 387 /var/spool/smb/netbudget.xls    21230 10:50:53
+ 
+****************************************************************************/
+static BOOL parse_lpq_lprng(char *line,print_queue_struct *buf,BOOL first)
+{
+#define        LPRNG_RANKTOK   0
+#define        LPRNG_USERTOK 1
+#define        LPRNG_PRIOTOK 2
+#define        LPRNG_JOBTOK    3
+#define        LPRNG_FILETOK   4
+#define        LPRNG_TOTALTOK 5
+#define LPRNG_TIMETOK 6
+#define        LPRNG_NTOK      7
+
+/****************************************************************************
+From lpd_status.c in LPRng source.
+0        1         2         3         4         5         6         7
+12345678901234567890123456789012345678901234567890123456789012345678901234 
+" Rank  Owner           Class Job Files                           Size Time"
+                        plp_snprintf( msg, sizeof(msg), "%-6s %-19s %c %03d %-32s",
+                                number, line, priority, cfp->number, error );
+                                plp_snprintf( msg + len, sizeof(msg)-len, "%4d",
+                                        cfp->jobsize );
+                                plp_snprintf( msg+len, sizeof(msg)-len, " %s",
+                                        Time_str( 1, cfp->statb.st_ctime ) );
+****************************************************************************/
+  /* The following define's are to be able to adjust the values if the
+LPRng source changes.  This is from version 2.3.0.  Magnus  */
+#define SPACE_W 1
+#define RANK_W 6
+#define OWNER_W 19
+#define CLASS_W 1
+#define JOB_W 3
+#define FILE_W 32
+/* The JOBSIZE_W is too small for big jobs, so time is pushed to the right */
+#define JOBSIZE_W 4
+ 
+#define RANK_POS 0
+#define OWNER_POS RANK_POS+RANK_W+SPACE_W
+#define CLASS_POS OWNER_POS+OWNER_W+SPACE_W
+#define JOB_POS CLASS_POS+CLASS_W+SPACE_W
+#define FILE_POS JOB_POS+JOB_W+SPACE_W
+#define JOBSIZE_POS FILE_POS+FILE_W
+
+  
+  string tok[LPRNG_NTOK];
+  int count=0;
+
+/* 
+Need to insert one space in front of the size, to be able to use
+next_token() unchanged.  I would have liked to be able to insert a
+space instead, to prevent losing that one char, but perl has spoiled
+me :-\  So I did it the easiest way.
+
+HINT: Use as short a path as possible for the samba spool directory.
+A long spool-path will just waste significant chars of the file name.
+*/
+
+  line[JOBSIZE_POS-1]=' ';
+
+  /* handle the case of "(stdin)" as a filename */
+  string_sub(line,"stdin","STDIN");
+  string_sub(line,"(","\"");
+  string_sub(line,")","\"");
+  
+  for (count=0; count<LPRNG_NTOK && next_token(&line,tok[count],NULL); count++) ;
+
+  /* we must get LPRNG_NTOK tokens */
+  if (count < LPRNG_NTOK)
+    return(False);
+
+  /* the Job and Total columns must be integer */
+  if (!isdigit(*tok[LPRNG_JOBTOK]) || !isdigit(*tok[LPRNG_TOTALTOK])) return(False);
+
+  /* if the fname contains a space then use STDIN */
+  /* I do not understand how this would be possible. Magnus. */
+  if (strchr(tok[LPRNG_FILETOK],' '))
+    strcpy(tok[LPRNG_FILETOK],"STDIN");
+
+  /* only take the last part of the filename */
+  {
+    string tmp;
+    char *p = strrchr(tok[LPRNG_FILETOK],'/');
+    if (p)
+      {
+       strcpy(tmp,p+1);
+       strcpy(tok[LPRNG_FILETOK],tmp);
+      }
+  }
+       
+
+  buf->job = atoi(tok[LPRNG_JOBTOK]);
+  buf->size = atoi(tok[LPRNG_TOTALTOK]);
+  buf->status = strequal(tok[LPRNG_RANKTOK],"active")?LPQ_PRINTING:LPQ_QUEUED;
+  /*  buf->time = time(NULL); */
+  buf->time = LPRng_time(tok,LPRNG_TIMETOK);
+DEBUG(3,("Time reported for job %d is %s", buf->job, ctime(&buf->time)));
+  StrnCpy(buf->user,tok[LPRNG_USERTOK],sizeof(buf->user)-1);
+  StrnCpy(buf->file,tok[LPRNG_FILETOK],sizeof(buf->file)-1);
+#ifdef LPRNG_PRIOTOK
+  /* Here I try to map the CLASS char to a number, but the number
+     is never shown in Print Manager under NT anyway... Magnus. */
+  buf->priority = atoi(tok[LPRNG_PRIOTOK-('A'-1)]);
+#else
+  buf->priority = 1;
+#endif
+  return(True);
+}
+
 
 
 /*******************************************************************
@@ -488,18 +640,19 @@ parse a lpq line
 here is an example of "lpstat -o dcslw" output under sysv
 
 dcslw-896               tridge            4712   Dec 20 10:30:30 on dcslw
+dcslw-897               tridge            4712   Dec 20 10:30:30 being held
 
 ****************************************************************************/
 static BOOL parse_lpq_sysv(char *line,print_queue_struct *buf,BOOL first)
 {
-  string tok[8];
+  string tok[9];
   int count=0;
   char *p;
 
   /* handle the dash in the job id */
   string_sub(line,"-"," ");
   
-  for (count=0; count<8 && next_token(&line,tok[count],NULL); count++) ;
+  for (count=0; count<9 && next_token(&line,tok[count],NULL); count++) ;
 
   /* we must get 7 tokens */
   if (count < 7)
@@ -520,7 +673,12 @@ static BOOL parse_lpq_sysv(char *line,print_queue_struct *buf,BOOL first)
 
   buf->job = atoi(tok[1]);
   buf->size = atoi(tok[3]);
-  buf->status = strequal(tok[7],"on")?LPQ_PRINTING:LPQ_QUEUED;
+  if (count > 7 && strequal(tok[7],"on"))
+    buf->status = LPQ_PRINTING;
+  else if (count > 8 && strequal(tok[7],"being") && strequal(tok[8],"held"))
+    buf->status = LPQ_PAUSED;
+  else
+    buf->status = LPQ_QUEUED;
   buf->priority = 0;
   buf->time = EntryTime(tok, 4, count, 7);
   StrnCpy(buf->user,tok[2],sizeof(buf->user)-1);
@@ -590,6 +748,76 @@ static BOOL parse_lpq_qnx(char *line,print_queue_struct *buf,BOOL first)
 }
 
 
+/****************************************************************************
+  parse a lpq line for the plp printing system
+  Bertrand Wallrich <Bertrand.Wallrich@loria.fr>
+
+redone by tridge. Here is a sample queue:
+
+Local  Printer 'lp2' (fjall):
+  Printing (started at Jun 15 13:33:58, attempt 1).
+    Rank Owner       Pr Opt  Job Host        Files           Size     Date
+  active tridge      X  -    6   fjall       /etc/hosts      739      Jun 15 13:33
+     3rd tridge      X  -    7   fjall       /etc/hosts      739      Jun 15 13:33
+
+****************************************************************************/
+static BOOL parse_lpq_plp(char *line,print_queue_struct *buf,BOOL first)
+{
+  string tok[11];
+  int count=0;
+
+  /* handle the case of "(standard input)" as a filename */
+  string_sub(line,"stdin","STDIN");
+  string_sub(line,"(","\"");
+  string_sub(line,")","\"");
+  
+  for (count=0; count<11 && next_token(&line,tok[count],NULL); count++) ;
+
+  /* we must get 11 tokens */
+  if (count < 11)
+    return(False);
+
+  /* the first must be "active" or begin with an integer */
+  if (strcmp(tok[0],"active") && !isdigit(tok[0][0]))
+    return(False);
+
+  /* the 5th and 8th must be integer */
+  if (!isdigit(*tok[4]) || !isdigit(*tok[7])) 
+    return(False);
+
+  /* if the fname contains a space then use STDIN */
+  if (strchr(tok[6],' '))
+    strcpy(tok[6],"STDIN");
+
+  /* only take the last part of the filename */
+  {
+    string tmp;
+    char *p = strrchr(tok[6],'/');
+    if (p)
+      {
+        strcpy(tmp,p+1);
+        strcpy(tok[6],tmp);
+      }
+  }
+
+
+  buf->job = atoi(tok[4]);
+
+  buf->size = atoi(tok[7]);
+  if (strchr(tok[7],'K'))
+    buf->size *= 1024;
+  if (strchr(tok[7],'M'))
+    buf->size *= 1024*1024;
+
+  buf->status = strequal(tok[0],"active")?LPQ_PRINTING:LPQ_QUEUED;
+  buf->priority = 0;
+  buf->time = time(NULL);
+  StrnCpy(buf->user,tok[1],sizeof(buf->user)-1);
+  StrnCpy(buf->file,tok[6],sizeof(buf->file)-1);
+  return(True);
+}
+
+
 
 char *stat0_strings[] = { "enabled", "online", "idle", "no entries", "free", "ready", NULL };
 char *stat1_strings[] = { "offline", "disabled", "down", "off", "waiting", "no daemon", NULL };
@@ -618,6 +846,12 @@ static BOOL parse_lpq_entry(int snum,char *line,
     case PRINT_QNX:
       ret = parse_lpq_qnx(line,buf,first);
       break;
+    case PRINT_LPRNG:
+      ret = parse_lpq_lprng(line,buf,first);
+      break;
+    case PRINT_PLP:
+      ret = parse_lpq_plp(line,buf,first);
+      break;
     default:
       ret = parse_lpq_bsd(line,buf,first);
       break;
@@ -633,6 +867,12 @@ static BOOL parse_lpq_entry(int snum,char *line,
   }
 #endif
 
+  /* We don't want the newline in the status message. */
+  {
+    char *p = strchr(line,'\n');
+    if (p) *p = 0;
+  }
+
   if (status && !ret)
     {
       /* a few simple checks to see if the line might be a
@@ -683,7 +923,6 @@ int get_printqueue(int snum,int cnum,print_queue_struct **queue,
   struct stat sbuf;
   BOOL dorun=True;
   int cachetime = lp_lpqcachetime();
-  int lfd = -1;
 
   *line = 0;
   check_lpq_cache(snum);
@@ -714,20 +953,10 @@ int get_printqueue(int snum,int cnum,print_queue_struct **queue,
 	DEBUG(3,("Using cached lpq output\n"));
 	dorun = False;
       }
-
-      if (dorun) {
-	lfd = file_lock(outfile,LPQ_LOCK_TIMEOUT);
-	if (lfd<0 || 
-	    (!fstat(lfd,&sbuf) && (time(NULL) - sbuf.st_mtime)<cachetime)) {
-	  DEBUG(3,("Using cached lpq output\n"));
-	  dorun = False;
-	  file_unlock(lfd); lfd = -1;
-	}
-      }
     }
 
   if (dorun) {
-    ret = smbrun(syscmd,outfile);
+    ret = smbrun(syscmd,outfile,True);
     DEBUG(3,("Running the command `%s' gave %d\n",syscmd,ret));
   }
 
@@ -735,7 +964,6 @@ int get_printqueue(int snum,int cnum,print_queue_struct **queue,
 
   f = fopen(outfile,"r");
   if (!f) {
-    if (lfd >= 0) file_unlock(lfd);
     return(0);
   }
 
@@ -766,12 +994,13 @@ int get_printqueue(int snum,int cnum,print_queue_struct **queue,
 
   fclose(f);
 
-  if (lfd >= 0) file_unlock(lfd);
-
-  if (!cachetime) 
+  if (!cachetime) {
     unlink(outfile);
-  else
+  } else {
+    /* we only expect this to succeed on trapdoor systems, on normal systems
+     the file is owned by root */
     chmod(outfile,0666);
+  }
   return(count);
 }
 
@@ -807,7 +1036,7 @@ void del_printqueue(int cnum,int snum,int jobid)
   string_sub(syscmd,"%j",jobstr);
   standard_sub(cnum,syscmd);
 
-  ret = smbrun(syscmd,NULL);
+  ret = smbrun(syscmd,NULL,False);
   DEBUG(3,("Running the command `%s' gave %d\n",syscmd,ret));  
   lpq_reset(snum); /* queue has changed */
 }
@@ -845,7 +1074,7 @@ void status_printjob(int cnum,int snum,int jobid,int status)
   string_sub(syscmd,"%j",jobstr);
   standard_sub(cnum,syscmd);
 
-  ret = smbrun(syscmd,NULL);
+  ret = smbrun(syscmd,NULL,False);
   DEBUG(3,("Running the command `%s' gave %d\n",syscmd,ret));  
   lpq_reset(snum); /* queue has changed */
 }
