@@ -25,13 +25,11 @@
 
 
 #include "includes.h"
-#include "loadparm.h"
 #include "trans2.h"
 
 /* look in server.c for some explanation of these variables */
 extern int Protocol;
 extern int DEBUGLEVEL;
-extern int chain_size;
 extern int maxxmit;
 extern int chain_fnum;
 extern char magic_char;
@@ -132,6 +130,44 @@ static int connection_error(char *inbuf,char *outbuf,int connection_num)
 }
 
 
+
+/****************************************************************************
+  parse a share descriptor string
+****************************************************************************/
+static void parse_connect(char *p,char *service,char *user,
+			  char *password,int *pwlen,char *dev)
+{
+  char *p2;
+
+  DEBUG(4,("parsing connect string %s\n",p));
+    
+  p2 = strrchr(p,'\\');
+  if (p2 == NULL)
+    strcpy(service,p);
+  else
+    strcpy(service,p2+1);
+  
+  p += strlen(p) + 2;
+  
+  strcpy(password,p);
+  *pwlen = strlen(password);
+
+  p += strlen(p) + 2;
+
+  strcpy(dev,p);
+  
+  *user = 0;
+  p = strchr(service,'%');
+  if (p != NULL)
+    {
+      *p = 0;
+      strcpy(user,p+1);
+    }
+}
+
+
+
+
 /****************************************************************************
   reply to a tcon
 ****************************************************************************/
@@ -145,13 +181,13 @@ int reply_tcon(char *inbuf,char *outbuf)
   int outsize = 0;
   int uid = SVAL(inbuf,smb_uid);
   int vuid;
-  int pwlen;
+  int pwlen=0;
 
   *service = *user = *password = *dev = 0;
 
   vuid = valid_uid(uid);
   
-  parse_connect(inbuf,service,user,password,&pwlen,dev);
+  parse_connect(smb_buf(inbuf)+1,service,user,password,&pwlen,dev);
 
   connection_num = make_connection(service,user,password,pwlen,dev,vuid);
   
@@ -179,12 +215,10 @@ int reply_tcon_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   pstring password;
   pstring devicename;
   int connection_num;
-  int outsize = 0;
   int uid = SVAL(inbuf,smb_uid);
   int vuid;
-  int smb_com2 = SVAL(inbuf,smb_vwv0);
-  int smb_off2 = SVAL(inbuf,smb_vwv1);
   int passlen = SVAL(inbuf,smb_vwv3);
+  BOOL doencrypt = SMBENCRYPT();
 
   *service = *user = *password = *devicename = 0;
 
@@ -198,8 +232,15 @@ int reply_tcon_and_X(char *inbuf,char *outbuf,int length,int bufsize)
     char *path;
     char *p;
     memcpy(password,smb_buf(inbuf),passlen);
-    password[passlen]=0;
+    password[passlen]=0;    
     path = smb_buf(inbuf) + passlen;
+
+    if (!doencrypt || passlen != 24) {
+      if (strequal(password," "))
+	*password = 0;
+      passlen = strlen(password);
+    }
+    
     DEBUG(4,("parsing net-path %s, passlen=%d\n",path,passlen));
     strcpy(service,path+2);
     p = strchr(service,'\\');
@@ -222,7 +263,7 @@ int reply_tcon_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   if (connection_num < 0)
     return(connection_error(inbuf,outbuf,connection_num));
 
-  outsize = set_message(outbuf,2,strlen(devicename)+1,True);
+  set_message(outbuf,2,strlen(devicename)+1,True);
   
   DEBUG(3,("%s tconX service=%s user=%s cnum=%d\n",timestring(),service,user,connection_num));
   
@@ -230,17 +271,9 @@ int reply_tcon_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   SSVAL(inbuf,smb_tid,connection_num);
   SSVAL(outbuf,smb_tid,connection_num);
 
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(chain_size + outsize)-4);
-
   strcpy(smb_buf(outbuf),devicename);
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
-
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -269,8 +302,13 @@ int reply_unknown(char *inbuf,char *outbuf)
 int reply_ioctl(char *inbuf,char *outbuf)
 {
   DEBUG(3,("ignoring ioctl\n"));
-  
+#if 0
+  /* we just say it succeeds and hope its all OK. 
+     some day it would be nice to interpret them individually */
+  return set_message(outbuf,1,0,True); 
+#else
   return(ERROR(ERRSRV,ERRnosupport));
+#endif
 }
 
 
@@ -279,28 +317,26 @@ reply to a session setup command
 ****************************************************************************/
 int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 {
-  int outsize = 0;
   int sess_uid;
   int gid;
-  int   smb_com2;
-  int   smb_off2;       
   int   smb_bufsize;    
   int   smb_mpxmax;     
   int   smb_vc_num;     
   uint32   smb_sesskey;    
-  int   smb_apasslen;   
+  int   smb_apasslen = 0;   
   pstring smb_apasswd;
   int   smb_ntpasslen = 0;   
   pstring smb_ntpasswd;
   BOOL valid_nt_password = False;
   pstring user;
   BOOL guest=False;
+  BOOL computer_id=False;
+  static BOOL done_sesssetup = False;
+  BOOL doencrypt = SMBENCRYPT();
 
   *smb_apasswd = 0;
   
   sess_uid = SVAL(inbuf,smb_uid);
-  smb_com2 = CVAL(inbuf,smb_vwv0);
-  smb_off2 = SVAL(inbuf,smb_vwv1);
   smb_bufsize = SVAL(inbuf,smb_vwv2);
   smb_mpxmax = SVAL(inbuf,smb_vwv3);
   smb_vc_num = SVAL(inbuf,smb_vwv4);
@@ -310,14 +346,17 @@ int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
     smb_apasslen = SVAL(inbuf,smb_vwv7);
     memcpy(smb_apasswd,smb_buf(inbuf),smb_apasslen);
     StrnCpy(user,smb_buf(inbuf)+smb_apasslen,sizeof(user)-1);
+
+    if (lp_security() != SEC_SERVER && !doencrypt)
+      smb_apasslen = strlen(smb_apasswd);
   } else {
     uint16 passlen1 = SVAL(inbuf,smb_vwv7);
     uint16 passlen2 = SVAL(inbuf,smb_vwv8);
-    BOOL doencrypt = SMBENCRYPT();
-    char *p = smb_buf(inbuf);
-    if (passlen1 > 256) passlen1 = 0;
-    if (passlen2 > 256) passlen2 = 0; /* I don't know why NT gives weird 
-					lengths sometimes */
+    char *p = smb_buf(inbuf);    
+
+    if (passlen1 != 24 && passlen2 != 24)
+      doencrypt = False;
+
     if(doencrypt) {
       /* Save the lanman2 password and the NT md4 password. */
       smb_apasslen = passlen1;
@@ -325,15 +364,35 @@ int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
       smb_ntpasslen = passlen2;
       memcpy(smb_ntpasswd,p+passlen1,smb_ntpasslen);
     } else {
-      /* for Win95 */
-      if (passlen1 > passlen2) {
-	smb_apasslen = passlen1;
-	StrnCpy(smb_apasswd,p,smb_apasslen);
-      } else {
-	smb_apasslen = passlen2;
-	StrnCpy(smb_apasswd,p + passlen1,smb_apasslen);
+      /* both Win95 and WinNT stuff up the password lengths for
+	 non-encrypting systems. Uggh. 
+      
+	 if passlen1==24 its a win95 system, and its setting the
+	 password length incorrectly. Luckily it still works with the
+	 default code because Win95 will null terminate the password
+	 anyway 
+
+	 if passlen1>0 and passlen2>0 then maybe its a NT box and its
+	 setting passlen2 to some random value which really stuffs
+	 things up. we need to fix that one.  */
+      if (passlen1 > 0 && passlen2 > 0 && passlen2 != 24 &&
+	  passlen2 != 1) {
+	passlen2 = 0;
+      }
+      /* we use the first password that they gave */
+      smb_apasslen = passlen1;
+      StrnCpy(smb_apasswd,p,smb_apasslen);      
+      
+      /* trim the password */
+      smb_apasslen = strlen(smb_apasswd);
+
+      /* wfwg sometimes uses a space instead of a null */
+      if (strequal(smb_apasswd," ")) {
+	smb_apasslen = 0;
+	*smb_apasswd = 0;
       }
     }
+    
     p += passlen1 + passlen2;
     strcpy(user,p); p = skip_string(p,1);
     DEBUG(3,("Domain=[%s]  NativeOS=[%s] NativeLanMan=[%s]\n",
@@ -343,6 +402,15 @@ int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 
   DEBUG(3,("sesssetupX:name=[%s]\n",user));
 
+  /* If name ends in $ then I think it's asking about whether a */
+  /* computer with that name (minus the $) has access. For now */
+  /* say yes to everything ending in $. */
+  if (user[strlen(user) - 1] == '$') {
+    computer_id = True;
+    user[strlen(user) - 1] = '\0';
+  }
+
+
   if (!*user)
     strcpy(user,lp_guestaccount(-1));
 
@@ -367,17 +435,22 @@ int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 	 first. This is superior as the passwords are mixed case 128 length unicode */
       if(smb_ntpasslen && !guest)
 	{
-	  if(!password_ok(user,smb_ntpasswd,smb_ntpasslen,NULL,True))
+	  if(!password_ok(user,smb_ntpasswd,smb_ntpasslen,NULL))
 	    DEBUG(0,("NT Password did not match ! Defaulting to Lanman\n"));
 	  else
 	    valid_nt_password = True;
 	} 
-      if (!valid_nt_password && !guest && !password_ok(user,smb_apasswd,smb_apasslen,NULL,False))
+      if (!valid_nt_password && !guest && !password_ok(user,smb_apasswd,smb_apasslen,NULL))
 	{
-#if !GUEST_SESSSETUP
-	  if (lp_security() >= SEC_USER && Get_Pwnam(user,True))
+	  if (!computer_id && lp_security() >= SEC_USER) {
+#if (GUEST_SESSSETUP == 0)
 	    return(ERROR(ERRSRV,ERRbadpw));
 #endif
+#if (GUEST_SESSSETUP == 1)
+	    if (Get_Pwnam(user,True))
+	      return(ERROR(ERRSRV,ERRbadpw));
+#endif
+	  }
  	  if (*smb_apasswd || !Get_Pwnam(user,True))
 	    strcpy(user,lp_guestaccount(-1));
 	  DEBUG(3,("Registered username %s for guest access\n",user));
@@ -403,15 +476,15 @@ int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 
   /* it's ok - setup a reply */
   if (Protocol < PROTOCOL_NT1) {
-    outsize = set_message(outbuf,3,0,True);
+    set_message(outbuf,3,0,True);
   } else {
     char *p;
-    outsize = set_message(outbuf,3,3,True);
+    set_message(outbuf,3,3,True);
     p = smb_buf(outbuf);
     strcpy(p,"Unix"); p = skip_string(p,1);
     strcpy(p,"Samba "); strcat(p,VERSION); p = skip_string(p,1);
-    strcpy(p,my_workgroup()); p = skip_string(p,1);
-    outsize = set_message(outbuf,3,PTR_DIFF(p,smb_buf(outbuf)),False);
+    strcpy(p,lp_workgroup()); p = skip_string(p,1);
+    set_message(outbuf,3,PTR_DIFF(p,smb_buf(outbuf)),False);
     /* perhaps grab OS version here?? */
   }
 
@@ -430,24 +503,19 @@ int reply_sesssetup_and_X(char *inbuf,char *outbuf,int length,int bufsize)
     SSVAL(inbuf,smb_uid,(uint16)pw->pw_uid);
   }
 
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(chain_size+outsize)-4);
-
-  if (guest)
+  if (guest && !computer_id)
     SSVAL(outbuf,smb_vwv2,1);
 
   /* register the name and uid as being validated, so further connections
      to a uid can get through without a password, on the same VC */
   register_uid(SVAL(inbuf,smb_uid),gid,user,guest);
  
-  maxxmit = MIN(maxxmit,smb_bufsize);
+  if (!done_sesssetup)
+    maxxmit = MIN(maxxmit,smb_bufsize);
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
+  done_sesssetup = True;
 
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -573,7 +641,7 @@ int reply_setatr(char *inbuf,char *outbuf)
   if (directory_exist(fname,NULL))
     mode |= aDIR;
   if (check_name(fname,cnum))
-    ok =  (dos_chmod(cnum,fname,mode) == 0);
+    ok =  (dos_chmod(cnum,fname,mode,NULL) == 0);
   if (ok)
     ok = set_filetime(fname,mtime);
   
@@ -939,6 +1007,10 @@ int reply_open(char *inbuf,char *outbuf)
   put_dos_date3(outbuf,smb_vwv2,mtime);
   SIVAL(outbuf,smb_vwv4,size);
   SSVAL(outbuf,smb_vwv6,rmode);
+
+  if (lp_fake_oplocks(SNUM(cnum))) {
+    CVAL(outbuf,smb_flg) |= (CVAL(inbuf,smb_flg) & (1<<5));
+  }
     
   return(outsize);
 }
@@ -952,12 +1024,10 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   pstring fname;
   int cnum = SVAL(inbuf,smb_tid);
   int fnum = -1;
-  int outsize = 0;
   int openmode = 0;
-  int smb_com2 = CVAL(inbuf,smb_vwv0);
-  int smb_off2 = SVAL(inbuf,smb_vwv1);
   int smb_mode = SVAL(inbuf,smb_vwv3);
   int smb_attr = SVAL(inbuf,smb_vwv5);
+  BOOL oplock_request = BITSETW(inbuf+smb_vwv2,1);
 #if 0
   int open_flags = SVAL(inbuf,smb_vwv2);
   int smb_sattr = SVAL(inbuf,smb_vwv4); 
@@ -969,6 +1039,10 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   struct stat sbuf;
   int smb_action = 0;
 
+  /* If it's an IPC, pass off the pipe handler. */
+  if (IS_IPC(cnum))
+    return reply_open_pipe_and_X(inbuf,outbuf,length,bufsize);
+
   /* XXXX we need to handle passed times, sattr and flags */
 
   strcpy(fname,smb_buf(inbuf));
@@ -987,7 +1061,7 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   if (!check_name(fname,cnum))
     return(UNIXERROR(ERRDOS,ERRnoaccess));
 
-  unixmode = unix_mode(cnum,smb_attr);
+  unixmode = unix_mode(cnum,smb_attr | aARCH);
       
   open_file_shared(fnum,cnum,fname,smb_mode,smb_ofun,unixmode,
 		   &rmode,&smb_action);
@@ -1008,9 +1082,11 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
     return(ERROR(ERRDOS,ERRnoaccess));
   }
 
-  outsize = set_message(outbuf,15,0,True);
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(chain_size+outsize)-4);
+  if (oplock_request && lp_fake_oplocks(SNUM(cnum))) {
+    smb_action |= (1<<15);
+  }
+
+  set_message(outbuf,15,0,True);
   SSVAL(outbuf,smb_vwv2,fnum);
   SSVAL(outbuf,smb_vwv3,fmode);
   put_dos_date3(outbuf,smb_vwv4,mtime);
@@ -1020,14 +1096,7 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 
   chain_fnum = fnum;
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
-
-  chain_fnum = -1;
-  
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -1036,26 +1105,25 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 ****************************************************************************/
 int reply_ulogoffX(char *inbuf,char *outbuf,int length,int bufsize)
 {
-  int outsize = 0;
-  int smb_com2 = CVAL(inbuf,smb_vwv0);
-  int smb_off2 = SVAL(inbuf,smb_vwv1);
   int uid = SVAL(inbuf,smb_uid);
 
   invalidate_uid(uid);
 
-  outsize = set_message(outbuf,2,0,True);
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(chain_size+outsize)-4);
+  /* in user level security we are supposed to close any files
+     open by this user */
+  if (lp_security() != SEC_SHARE) {
+    int i;
+    for (i=0;i<MAX_OPEN_FILES;i++)
+      if (Files[i].uid == uid && Files[i].open) {
+	close_file(i);
+      }
+  }
+
+  set_message(outbuf,2,0,True);
 
   DEBUG(3,("%s ulogoffX uid=%d\n",timestring(),uid));
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
-
-  
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -1102,6 +1170,10 @@ int reply_mknew(char *inbuf,char *outbuf)
   
   outsize = set_message(outbuf,1,0,True);
   SSVAL(outbuf,smb_vwv0,fnum);
+
+  if (lp_fake_oplocks(SNUM(cnum))) {
+    CVAL(outbuf,smb_flg) |= (CVAL(inbuf,smb_flg) & (1<<5));
+  }
   
   DEBUG(2,("new file %s\n",fname));
   DEBUG(3,("%s mknew %s fd=%d fnum=%d cnum=%d dmode=%d umode=%o\n",timestring(),fname,Files[fnum].fd,fnum,cnum,createmode,unixmode));
@@ -1148,6 +1220,10 @@ int reply_ctemp(char *inbuf,char *outbuf)
   SSVAL(outbuf,smb_vwv0,fnum);
   CVAL(smb_buf(outbuf),0) = 4;
   strcpy(smb_buf(outbuf) + 1,fname2);
+
+  if (lp_fake_oplocks(SNUM(cnum))) {
+    CVAL(outbuf,smb_flg) |= (CVAL(inbuf,smb_flg) & (1<<5));
+  }
   
   DEBUG(2,("created temp file %s\n",fname2));
   DEBUG(3,("%s ctemp %s fd=%d fnum=%d cnum=%d dmode=%d umode=%o\n",timestring(),fname2,Files[fnum].fd,fnum,cnum,createmode,unixmode));
@@ -1169,7 +1245,9 @@ static BOOL can_delete(char *fname,int cnum,int dirtype)
   if (sys_lstat(fname,&sbuf) != 0) return(False);
   fmode = dos_mode(cnum,fname,&sbuf);
   if (fmode & aDIR) return(False);
-  if (fmode & aRONLY) return(False);
+  if (!lp_delete_readonly(SNUM(cnum))) {
+    if (fmode & aRONLY) return(False);
+  }
   if ((fmode & ~dirtype) & (aHIDDEN | aSYSTEM))
     return(False);
   if (!check_file_sharing(cnum,fname)) return(False);
@@ -1231,6 +1309,11 @@ int reply_unlink(char *inbuf,char *outbuf)
     if (check_name(directory,cnum))
       dirptr = OpenDir(directory);
 
+    /* XXXX the CIFS spec says that if bit0 of the flags2 field is set then
+       the pattern matches against the long name, otherwise the short name 
+       We don't implement this yet XXXX
+       */
+
     if (dirptr)
       {
 	error = ERRbadfile;
@@ -1350,7 +1433,7 @@ int reply_readbraw(char *inbuf, char *outbuf)
 	     fname,startpos,nread,ret));
 
 #else
-  ret = read_file(fnum,header+4,startpos,nread,nread,-1,False);
+  ret = read_file(fnum,header+4,startpos,nread);
   if (ret < mincount) ret = 0;
 
   _smb_setlen(header,ret);
@@ -1392,7 +1475,7 @@ int reply_lockread(char *inbuf,char *outbuf)
   if(!do_lock( fnum, cnum, numtoread, startpos, &eclass, &ecode))
     return (ERROR(eclass,ecode));
 
-  nread = read_file(fnum,data,startpos,numtoread,numtoread,-1,False);
+  nread = read_file(fnum,data,startpos,numtoread);
   
   if (nread < 0)
     return(UNIXERROR(ERRDOS,ERRnoaccess));
@@ -1437,7 +1520,7 @@ int reply_read(char *inbuf,char *outbuf)
     return(ERROR(ERRDOS,ERRlock));	
 
   if (numtoread > 0)
-    nread = read_file(fnum,data,startpos,numtoread,numtoread,-1,False);
+    nread = read_file(fnum,data,startpos,numtoread);
   
   if (nread < 0)
     return(UNIXERROR(ERRDOS,ERRnoaccess));
@@ -1459,8 +1542,6 @@ int reply_read(char *inbuf,char *outbuf)
 ****************************************************************************/
 int reply_read_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 {
-  int smb_com2 = CVAL(inbuf,smb_vwv0);
-  int smb_off2 = SVAL(inbuf,smb_vwv1);
   int fnum = GETFNUM(inbuf,smb_vwv2);
   uint32 smb_offs = IVAL(inbuf,smb_vwv3);
   int smb_maxcnt = SVAL(inbuf,smb_vwv5);
@@ -1468,7 +1549,6 @@ int reply_read_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   int cnum;
   int nread = -1;
   char *data;
-  int outsize = 0;
   BOOL ok = False;
 
   cnum = SVAL(inbuf,smb_tid);
@@ -1477,38 +1557,28 @@ int reply_read_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   CHECK_READ(fnum);
   CHECK_ERROR(fnum);
 
-  outsize = set_message(outbuf,12,0,True);
+  set_message(outbuf,12,0,True);
   data = smb_buf(outbuf);
 
   if (is_locked(fnum,cnum,smb_maxcnt,smb_offs))
     return(ERROR(ERRDOS,ERRlock));
-  nread = read_file(fnum,data,smb_offs,smb_maxcnt,smb_maxcnt,-1,False);
+  nread = read_file(fnum,data,smb_offs,smb_maxcnt);
   ok = True;
   
   if (nread < 0)
     return(UNIXERROR(ERRDOS,ERRnoaccess));
   
-  outsize += nread;
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(outsize+chain_size)-4);
   SSVAL(outbuf,smb_vwv5,nread);
-  SSVAL(outbuf,smb_vwv6,smb_offset(data,outbuf) + chain_size);
+  SSVAL(outbuf,smb_vwv6,smb_offset(data,outbuf));
   SSVAL(smb_buf(outbuf),-2,nread);
   
-  DEBUG(3,("%s readX fnum=%d cnum=%d min=%d max=%d nread=%d com2=%d off2=%d\n",
+  DEBUG(3,("%s readX fnum=%d cnum=%d min=%d max=%d nread=%d\n",
 	timestring(),fnum,cnum,
-	smb_mincnt,smb_maxcnt,nread,smb_com2,smb_off2));
+	smb_mincnt,smb_maxcnt,nread));
 
   chain_fnum = fnum;
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
-  
-  chain_fnum = -1;
-  
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -1559,7 +1629,7 @@ int reply_writebraw(char *inbuf,char *outbuf)
     DEBUG(0,("couldn't seek to %d in writebraw\n",startpos));
 
   if (numtowrite>0)
-    nwritten = write_data(Files[fnum].fd,data,numtowrite);
+    nwritten = write_file(fnum,data,numtowrite);
   
   DEBUG(3,("%s writebraw1 fnum=%d cnum=%d start=%d num=%d wrote=%d sync=%d\n",
 	   timestring(),fnum,cnum,startpos,numtowrite,nwritten,write_through));
@@ -1577,7 +1647,7 @@ int reply_writebraw(char *inbuf,char *outbuf)
   send_smb(Client,outbuf);
   
   /* Now read the raw data into the buffer and write it */
-  if(read_smb_length(Client,inbuf,0) == -1) {
+  if (read_smb_length(Client,inbuf,SMB_SECONDARY_WAIT) == -1) {
     exit_server("secondary writebraw failed");
   }
   
@@ -1627,7 +1697,6 @@ int reply_writeunlock(char *inbuf,char *outbuf)
   int cnum,fnum;
   int nwritten = -1;
   int outsize = 0;
-  int fd;
   char *data;
   uint32 numtowrite,startpos;
   int eclass;
@@ -1644,8 +1713,6 @@ int reply_writeunlock(char *inbuf,char *outbuf)
   startpos = IVAL(inbuf,smb_vwv2);
   data = smb_buf(inbuf) + 3;
   
-  fd = Files[fnum].fd;
-
   if (is_locked(fnum,cnum,numtowrite,startpos))
     return(ERROR(ERRDOS,ERRlock));
 
@@ -1657,7 +1724,7 @@ int reply_writeunlock(char *inbuf,char *outbuf)
   if(numtowrite == 0)
     nwritten = 0;
   else
-    nwritten = write_data(fd,data,numtowrite);
+    nwritten = write_file(fnum,data,numtowrite);
   
   if (lp_syncalways(SNUM(cnum)))
     sync_file(fnum);
@@ -1688,7 +1755,6 @@ int reply_write(char *inbuf,char *outbuf,int dum1,int dum2)
   int nwritten = -1;
   int outsize = 0;
   int startpos;
-  int fd;
   char *data;
 
   dum1 = dum2 = 0;
@@ -1705,8 +1771,6 @@ int reply_write(char *inbuf,char *outbuf,int dum1,int dum2)
   startpos = IVAL(inbuf,smb_vwv2);
   data = smb_buf(inbuf) + 3;
   
-  fd = Files[fnum].fd;
-
   if (is_locked(fnum,cnum,numtowrite,startpos))
     return(ERROR(ERRDOS,ERRlock));
 
@@ -1716,9 +1780,9 @@ int reply_write(char *inbuf,char *outbuf,int dum1,int dum2)
      zero then the file size should be extended or
      truncated to the size given in smb_vwv[2-3] */
   if(numtowrite == 0)
-    nwritten = set_filelen(fd, startpos);
+    nwritten = set_filelen(Files[fnum].fd, startpos);
   else
-    nwritten = write_data(fd,data,numtowrite);
+    nwritten = write_file(fnum,data,numtowrite);
   
   if (lp_syncalways(SNUM(cnum)))
     sync_file(fnum);
@@ -1746,8 +1810,6 @@ int reply_write(char *inbuf,char *outbuf,int dum1,int dum2)
 ****************************************************************************/
 int reply_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 {
-  int smb_com2 = CVAL(inbuf,smb_vwv0);
-  int smb_off2 = SVAL(inbuf,smb_vwv1);
   int fnum = GETFNUM(inbuf,smb_vwv2);
   uint32 smb_offs = IVAL(inbuf,smb_vwv3);
   int smb_dsize = SVAL(inbuf,smb_vwv10);
@@ -1755,8 +1817,6 @@ int reply_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   BOOL write_through = BITSETW(inbuf+smb_vwv7,0);
   int cnum;
   int nwritten = -1;
-  int outsize = 0;
-  int fd;
   char *data;
 
   cnum = SVAL(inbuf,smb_tid);
@@ -1767,8 +1827,6 @@ int reply_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
 
   data = smb_base(inbuf) + smb_doff;
 
-  fd = Files[fnum].fd;
-
   if (is_locked(fnum,cnum,smb_dsize,smb_offs))
     return(ERROR(ERRDOS,ERRlock));
 
@@ -1781,15 +1839,13 @@ int reply_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   if(smb_dsize == 0)
     nwritten = 0;
   else
-    nwritten = write_data(fd,data,smb_dsize);
+    nwritten = write_file(fnum,data,smb_dsize);
   
   if(((nwritten == 0) && (smb_dsize != 0))||(nwritten < 0))
     return(UNIXERROR(ERRDOS,ERRnoaccess));
 
-  outsize = set_message(outbuf,6,0,True);
+  set_message(outbuf,6,0,True);
   
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(outsize+chain_size)-4);
   SSVAL(outbuf,smb_vwv2,nwritten);
   
   if (nwritten < smb_dsize) {
@@ -1804,14 +1860,7 @@ int reply_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   if (lp_syncalways(SNUM(cnum)) || write_through)
     sync_file(fnum);
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
-  
-  chain_fnum = -1;
-  
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -1923,11 +1972,11 @@ int reply_close(char *inbuf,char *outbuf)
 
   mtime = make_unix_date3(inbuf+smb_vwv1);
 
-  close_file(fnum);
-
   /* try and set the date */
   set_filetime(Files[fnum].name,mtime);
 
+  close_file(fnum);
+
   /* We have a cached error */
   if(eclass || err)
     return(ERROR(eclass,err));
@@ -1969,12 +2018,12 @@ int reply_writeclose(char *inbuf,char *outbuf)
       
   seek_file(fnum,startpos);
       
-  nwritten = write_data(Files[fnum].fd,data,numtowrite);
-
-  close_file(fnum);
+  nwritten = write_file(fnum,data,numtowrite);
 
   set_filetime(Files[fnum].name,mtime);
   
+  close_file(fnum);
+
   DEBUG(3,("%s writeclose fnum=%d cnum=%d num=%d wrote=%d (numopen=%d)\n",
 	   timestring(),fnum,cnum,numtowrite,nwritten,
 	   Connections[cnum].num_files_open));
@@ -2058,6 +2107,11 @@ int reply_tdis(char *inbuf,char *outbuf)
   cnum = SVAL(inbuf,smb_tid);
   uid = SVAL(inbuf,smb_uid);
 
+  if (!OPEN_CNUM(cnum)) {
+    DEBUG(4,("Invalid cnum in tdis (%d)\n",cnum));
+    return(ERROR(ERRSRV,ERRinvnid));
+  }
+
   Connections[cnum].used = False;
 
   close_cnum(cnum,uid);
@@ -2316,7 +2370,7 @@ int reply_printwrite(char *inbuf,char *outbuf)
   numtowrite = SVAL(smb_buf(inbuf),1);
   data = smb_buf(inbuf) + 3;
   
-  if (write_data(Files[fnum].fd,data,numtowrite) != numtowrite)
+  if (write_file(fnum,data,numtowrite) != numtowrite)
     return(UNIXERROR(ERRDOS,ERRnoaccess));
   
   DEBUG(3,("%s printwrite fnum=%d cnum=%d num=%d\n",timestring(),fnum,cnum,numtowrite));
@@ -2580,7 +2634,7 @@ static BOOL copy_file(char *src,char *dest1,int cnum,int ofun,
 {
   int Access,action;
   struct stat st;
-  int ret;
+  int ret=0;
   int fnum1,fnum2;
   pstring dest;
   
@@ -2805,8 +2859,6 @@ int reply_setdir(char *inbuf,char *outbuf)
 ****************************************************************************/
 int reply_lockingX(char *inbuf,char *outbuf,int length,int bufsize)
 {
-  int smb_com2 = CVAL(inbuf,smb_vwv0);
-  int smb_off2 = SVAL(inbuf,smb_vwv1);
   int fnum = GETFNUM(inbuf,smb_vwv2);
   uint16 locktype = SVAL(inbuf,smb_vwv3);
   uint16 num_ulocks = SVAL(inbuf,smb_vwv6);
@@ -2817,7 +2869,7 @@ int reply_lockingX(char *inbuf,char *outbuf,int length,int bufsize)
   int i;
   char *data;
   uint32 ecode=0, dummy2;
-  int outsize, eclass=0, dummy1;
+  int eclass=0, dummy1;
 
   cnum = SVAL(inbuf,smb_tid);
 
@@ -2856,24 +2908,14 @@ int reply_lockingX(char *inbuf,char *outbuf,int length,int bufsize)
     return ERROR(eclass,ecode);
   }
 
-  outsize = set_message(outbuf,2,0,True);
-  
-  CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,(outsize+chain_size)-4);
+  set_message(outbuf,2,0,True);
   
   DEBUG(3,("%s lockingX fnum=%d cnum=%d type=%d num_locks=%d num_ulocks=%d\n",
 	timestring(),fnum,cnum,locktype,num_locks,num_ulocks));
 
   chain_fnum = fnum;
 
-  if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
-			   outbuf,outbuf+outsize,
-			   length,bufsize);
-  
-  chain_fnum = -1;
-  
-  return(outsize);
+  return chain_reply(inbuf,outbuf,length,bufsize);
 }
 
 
@@ -2925,7 +2967,7 @@ int reply_readbmpx(char *inbuf,char *outbuf,int length,int bufsize)
     {
       int N = MIN(max_per_packet,tcount-total_read);
   
-      nread = read_file(fnum,data,startpos,N,N,-1,False);
+      nread = read_file(fnum,data,startpos,N);
 
       if (nread <= 0) nread = 0;
 
@@ -2959,7 +3001,6 @@ int reply_writebmpx(char *inbuf,char *outbuf)
   int outsize = 0;
   int32 startpos;
   int tcount, write_through, smb_doff;
-  int fd;
   char *data;
   
   cnum = SVAL(inbuf,smb_tid);
@@ -2981,13 +3022,11 @@ int reply_writebmpx(char *inbuf,char *outbuf)
      not an SMBwritebmpx - set this up now so we don't forget */
   CVAL(outbuf,smb_com) = SMBwritec;
 
-  fd = Files[fnum].fd;
-
   if (is_locked(fnum,cnum,tcount,startpos))
     return(ERROR(ERRDOS,ERRlock));
 
   seek_file(fnum,startpos);
-  nwritten = write_data(fd,data,numtowrite);
+  nwritten = write_file(fnum,data,numtowrite);
 
   if(lp_syncalways(SNUM(cnum)) || write_through)
     sync_file(fnum);
@@ -3055,7 +3094,6 @@ int reply_writebs(char *inbuf,char *outbuf)
   int outsize = 0;
   int32 startpos;
   int tcount, write_through, smb_doff;
-  int fd;
   char *data;
   write_bmpx_struct *wbms;
   BOOL send_response = False;
@@ -3088,10 +3126,8 @@ int reply_writebs(char *inbuf,char *outbuf)
   if(wbms->wr_discard)
     return -1; /* Just discard the packet */
 
-  fd = Files[fnum].fd;
-
   seek_file(fnum,startpos);
-  nwritten = write_data(fd,data,numtowrite);
+  nwritten = write_file(fnum,data,numtowrite);
 
   if(lp_syncalways(SNUM(cnum)) || write_through)
     sync_file(fnum);
@@ -3181,7 +3217,7 @@ int reply_getattrE(char *inbuf,char *outbuf)
   CHECK_ERROR(fnum);
 
   /* Do an fstat on this file */
-  if(fstat( Files[fnum].fd, &sbuf))
+  if(fstat(Files[fnum].fd, &sbuf))
     return(UNIXERROR(ERRDOS,ERRnoaccess));
   
   mode = dos_mode(cnum,Files[fnum].name,&sbuf);
