@@ -22,25 +22,11 @@
 */
 
 #include "includes.h"
-#include "loadparm.h"
 #include "des.h"
 #include "md4.h"
 
 extern int DEBUGLEVEL;
 
-#ifndef uchar
-#define uchar unsigned char
-#endif
-#ifndef int16
-#define int16 unsigned short
-#endif
-#ifndef uint16
-#define uint16 unsigned short
-#endif
-#ifndef uint32
-#define uint32 unsigned int
-#endif
-
 #include "byteorder.h"
 
 void str_to_key(uchar *str,uchar *key)
@@ -68,8 +54,12 @@ void D1(uchar *k, uchar *d, uchar *out)
   des_cblock deskey;
 
   str_to_key(k,(uchar *)deskey);
-  des_set_key(deskey,ks);
-  des_ecb_encrypt(d, out, ks, DES_DECRYPT);
+#ifdef __FreeBSD__
+  des_set_key(&deskey,ks);
+#else /* __FreeBSD__ */
+  des_set_key((des_cblock *)deskey,ks);
+#endif /* __FreeBsd */
+  des_ecb_encrypt((des_cblock *)d,(des_cblock *)out, ks, DES_DECRYPT);
 }
 
 void E1(uchar *k, uchar *d, uchar *out)
@@ -78,8 +68,12 @@ void E1(uchar *k, uchar *d, uchar *out)
   des_cblock deskey;
 
   str_to_key(k,(uchar *)deskey);
-  des_set_key(deskey,ks);
-  des_ecb_encrypt(d, out, ks, DES_ENCRYPT);
+#ifdef __FreeBSD__
+  des_set_key(&deskey,ks);
+#else /* __FreeBsd__ */
+  des_set_key((des_cblock *)deskey,ks);
+#endif /* __FreeBsd__ */
+  des_ecb_encrypt((des_cblock *)d,(des_cblock *)out, ks, DES_ENCRYPT);
 }
  
 void E_P16(uchar *p14,uchar *p16)
@@ -119,7 +113,7 @@ void SMBencrypt(uchar *passwd, uchar *c8, uchar *p24)
   StrnCpy((char *)p14,(char *)passwd,14);
 
   strupper((char *)p14);
-  E_P16(p14, p21);
+  E_P16(p14, p21); 
   E_P24(p21, c8, p24);
 }
 
@@ -166,7 +160,7 @@ void E_md4hash(uchar *passwd, uchar *p16)
 	MDstruct MD;
  
 	/* Password cannot be longer than 128 characters */
-	len = strlen(passwd);
+	len = strlen((char *)passwd);
 	if(len > 128)
 		len = 128;
 	/* Password must be converted to NT unicode */
@@ -198,5 +192,5 @@ void SMBNTencrypt(uchar *passwd, uchar *c8, uchar *p24)
 }
 
 #else
-void smbencrypt_dummy(void){}
+ void smbencrypt_dummy(void){}
 #endif
