@@ -21,15 +21,21 @@
 
 #include "includes.h"
 
+extern int DEBUGLEVEL;
+
 /*
    The idea is that this file will eventually have wrappers around all
-   important system calls in samba. The aim is twofold:
+   important system calls in samba. The aims are:
 
    - to enable easier porting by putting OS dependent stuff in here
 
    - to allow for hooks into other "pseudo-filesystems"
 
    - to allow easier integration of things like the japanese extensions
+
+   - to support the philosophy of Samba to expose the features of
+     the OS within the SMB model. In general whatever file/printer/variable
+     expansions/etc make sense to the OS should be acceptable to Samba.
 */
 
 
@@ -45,13 +51,9 @@ static int pollfd(int fd)
 
 #ifdef HAS_RDCHK
   r = rdchk(fd);
-#endif
-
-#ifdef HAS_TCRDCHK
+#elif defined(TCRDCHK)
   (void)ioctl(fd, TCRDCHK, &r);
-#endif
-
-#ifdef HAS_FIONREAD
+#else
   (void)ioctl(fd, FIONREAD, &r);
 #endif
 
@@ -81,7 +83,7 @@ int sys_select(fd_set *fds,struct timeval *tval)
 	return(found);
       }
       
-      if (tval && tval.tv_sec < counter) return(0);
+      if (tval && tval->tv_sec < counter) return(0);
       sleep(1);
       counter++;
     }
@@ -96,7 +98,7 @@ int sys_select(fd_set *fds,struct timeval *tval)
   do {
     if (tval) memcpy((void *)&t2,(void *)tval,sizeof(t2));
     errno = 0;
-    selrtn = select(FD_SETSIZE,SELECT_CAST fds,NULL,NULL,tval?&t2:NULL);
+    selrtn = select(255,SELECT_CAST fds,NULL,NULL,tval?&t2:NULL);
   } while (selrtn<0 && errno == EINTR);
 
   return(selrtn);
@@ -139,6 +141,14 @@ int sys_stat(char *fname,struct stat *sbuf)
   return(stat(dos_to_unix(fname,False),sbuf));
 }
 
+/*******************************************************************
+The wait() calls vary between systems
+********************************************************************/
+int sys_waitpid(pid_t pid,int *status,int options)
+{
+  return waitpid(pid,status,options);
+}
+
 /*******************************************************************
 don't forget lstat()
 ********************************************************************/
@@ -188,16 +198,19 @@ for rename()
 ********************************************************************/
 int sys_rename(char *from, char *to)
 {
-#ifdef KANJI
     pstring zfrom, zto;
     strcpy (zfrom, dos_to_unix (from, False));
     strcpy (zto, dos_to_unix (to, False));
     return rename (zfrom, zto);
-#else 
-    return rename (from, to);
-#endif /* KANJI */
 }
 
+/*******************************************************************
+for chmod
+********************************************************************/
+int sys_chmod(char *fname,int mode)
+{
+  return(chmod(dos_to_unix(fname,False),mode));
+}
 
 /*******************************************************************
 chown isn't used much but OS/2 doesn't have it
