@@ -22,7 +22,6 @@
 */
 
 #include "includes.h"
-#include "loadparm.h"
 #include "trans2.h"
 
 extern int DEBUGLEVEL;
@@ -30,7 +29,7 @@ extern int Protocol;
 extern connection_struct Connections[];
 extern files_struct Files[];
 extern BOOL case_sensitive;
-
+extern int Client;
 
 /****************************************************************************
   Send the required number of replies back.
@@ -54,6 +53,7 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
   char *pp = params;
   char *pd = pdata;
   int params_sent_thistime, data_sent_thistime, total_sent_thistime;
+  int alignment_offset = 1;
 
   /* Initially set the wcnt area to be 10 - this is true for all
      trans2 replies */
@@ -63,18 +63,20 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
      the empty packet */
   if(params_to_send == 0 && data_to_send == 0)
     {
-      send_smb(outbuf);
+      send_smb(Client,outbuf);
       return 0;
     }
 
   /* Space is bufsize minus Netbios over TCP header minus SMB header */
-  useable_space = bufsize - (smb_buf(outbuf) - outbuf);
-  useable_space = MIN(useable_space, maxxmit);
+  /* The + 1 is to align the param and data bytes on an even byte
+     boundary. NT 4.0 Beta needs this to work correctly. */
+  useable_space = bufsize - ((smb_buf(outbuf)+alignment_offset) - outbuf);
+  useable_space = MIN(useable_space, maxxmit); /* XXX is this needed? correct? */
 
   while( params_to_send || data_to_send)
     {
       /* Calculate whether we will totally or partially fill this packet */
-      total_sent_thistime = params_to_send + data_to_send;
+      total_sent_thistime = params_to_send + data_to_send + alignment_offset;
       total_sent_thistime = MIN(total_sent_thistime, useable_space);
 
       set_message(outbuf, 10, total_sent_thistime, True);
@@ -100,7 +102,7 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
 	     parameter bytes, however the first 4 bytes of outbuf are
 	     the Netbios over TCP header. Thus use smb_base() to subtract
 	     them from the calculation */
-	  SSVAL(outbuf,smb_proff,(smb_buf(outbuf) - smb_base(outbuf)));
+	  SSVAL(outbuf,smb_proff,((smb_buf(outbuf)+alignment_offset) - smb_base(outbuf)));
 	  /* Absolute displacement of param bytes sent in this packet */
 	  SSVAL(outbuf,smb_prdisp,pp - params);
 	}
@@ -113,16 +115,17 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
 	} else {
 	  /* The offset of the data bytes is the offset of the
 	     parameter bytes plus the number of parameters being sent this time */
-	  SSVAL(outbuf,smb_droff,(smb_buf(outbuf) - smb_base(outbuf)) + params_sent_thistime);
+	  SSVAL(outbuf,smb_droff,((smb_buf(outbuf)+alignment_offset) - 
+				  smb_base(outbuf)) + params_sent_thistime);
 	  SSVAL(outbuf,smb_drdisp, pd - pdata);
 	}
 
       /* Copy the param bytes into the packet */
       if(params_sent_thistime)
-	memcpy(smb_buf(outbuf),pp,params_sent_thistime);
+	memcpy((smb_buf(outbuf)+alignment_offset),pp,params_sent_thistime);
       /* Copy in the data bytes */
       if(data_sent_thistime)
-	memcpy(smb_buf(outbuf)+params_sent_thistime,pd,data_sent_thistime);
+	memcpy(smb_buf(outbuf)+alignment_offset+params_sent_thistime,pd,data_sent_thistime);
 
       DEBUG(9,("t2_rep: params_sent_thistime = %d, data_sent_thistime = %d, useable_space = %d\n",
 	       params_sent_thistime, data_sent_thistime, useable_space));
@@ -130,7 +133,7 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
 	       params_to_send, data_to_send, paramsize, datasize));
 
       /* Send the packet */
-      send_smb(outbuf);
+      send_smb(Client,outbuf);
 
       pp += params_sent_thistime;
       pd += data_sent_thistime;
@@ -160,6 +163,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
   char *params = *pparams;
   int16 open_mode = SVAL(params, 2);
   int16 open_attr = SVAL(params,6);
+  BOOL oplock_request = BITSETW(params,1);
 #if 0
   BOOL return_additional_info = BITSETW(params,0);
   int16 open_sattr = SVAL(params, 4);
@@ -194,7 +198,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
   if (!check_name(fname,cnum))
     return(UNIXERROR(ERRDOS,ERRnoaccess));
 
-  unixmode = unix_mode(cnum,open_attr);
+  unixmode = unix_mode(cnum,open_attr | aARCH);
       
       
   open_file_shared(fnum,cnum,fname,open_mode,open_ofun,unixmode,
@@ -229,6 +233,10 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
   SIVAL(params,8, size);
   SSVAL(params,12,rmode);
 
+  if (oplock_request && lp_fake_oplocks(SNUM(cnum))) {
+    smb_action |= (1<<15);
+  }
+
   SSVAL(params,18,smb_action);
   SIVAL(params,20,inode);
  
@@ -258,12 +266,13 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
   char *p, *pdata = *ppdata;
   int reskey=0, prev_dirpos=0;
   int mode=0;
-  uint32 size=0;
+  uint32 size=0,len;
   uint32 mdate=0, adate=0, cdate=0;
-  char *name_ptr;
+  char *nameptr;
   BOOL isrootdir = (strequal(Connections[cnum].dirpath,"./") ||
 		    strequal(Connections[cnum].dirpath,".") ||
 		    strequal(Connections[cnum].dirpath,"/"));
+  BOOL was_8_3;
 
   *fname = 0;
   *out_of_space = False;
@@ -320,11 +329,11 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
 
 	  mode = dos_mode(cnum,pathreal,&sbuf);
 
-	  if (((mode & ~dirtype) & (aHIDDEN | aSYSTEM | aDIR)) != 0)
-	    {	      
-	      DEBUG(5,("[%s] attribs didn't match %x\n",fname,dirtype));
-	      continue;
-	    }
+	  if (!dir_check_ftype(cnum,mode,&sbuf,dirtype)) {
+	    DEBUG(5,("[%s] attribs didn't match %x\n",fname,dirtype));
+	    continue;
+	  }
+
 	  size = sbuf.st_size;
 	  mdate = sbuf.st_mtime;
 	  adate = sbuf.st_atime;
@@ -344,21 +353,9 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
 #endif
 
   p = pdata;
-  name_ptr = p;
+  nameptr = p;
 
-#ifdef MANGLE_LONG_FILENAMES
-  {
-    BOOL illegal = False;
-    int i;
-    int l = strlen(fname);
-    for (i=0;i<l;i++) 
-      if (strchr("*\\/?<>|\":{}",fname[i])) {
-	illegal = True;
-	break;
-      }
-    name_map_mangle(fname,illegal,SNUM(cnum));
-  }
-#endif
+  name_map_mangle(fname,False,SNUM(cnum));
 
   switch (info_level)
     {
@@ -371,11 +368,11 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       put_dos_date2(p,l1_fdateLastAccess,adate);
       put_dos_date2(p,l1_fdateLastWrite,mdate);
       SIVAL(p,l1_cbFile,size);
-      SIVAL(p,l1_cbFileAlloc,1024); /* Make 1024 for now */
+      SIVAL(p,l1_cbFileAlloc,ROUNDUP(size,1024));
       SSVAL(p,l1_attrFile,mode);
       SCVAL(p,l1_cchName,strlen(fname));
       strcpy(p + l1_achName, fname);
-      name_ptr = p + l1_achName;
+      nameptr = p + l1_achName;
       p += l1_achName + strlen(fname) + 1;
       break;
 
@@ -389,12 +386,12 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       put_dos_date2(p,l2_fdateLastAccess,adate);
       put_dos_date2(p,l2_fdateLastWrite,mdate);
       SIVAL(p,l2_cbFile,size);
-      SIVAL(p,l2_cbFileAlloc,1024); /* Make 1024 for now */
+      SIVAL(p,l2_cbFileAlloc,ROUNDUP(size,1024));
       SSVAL(p,l2_attrFile,mode);
       SIVAL(p,l2_cbList,0); /* No extended attributes */
       SCVAL(p,l2_cchName,strlen(fname));
       strcpy(p + l2_achName, fname);
-      name_ptr = p + l2_achName;
+      nameptr = p + l2_achName;
       p += l2_achName + strlen(fname) + 1;
       break;
 
@@ -404,12 +401,12 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       put_dos_date2(p,8,adate);
       put_dos_date2(p,12,mdate);
       SIVAL(p,16,size);
-      SIVAL(p,20,1024);
+      SIVAL(p,20,ROUNDUP(size,1024));
       SSVAL(p,24,mode);
       SIVAL(p,26,4);
       CVAL(p,30) = strlen(fname);
       strcpy(p+31, fname);
-      name_ptr = p+31;
+      nameptr = p+31;
       p += 31 + strlen(fname) + 1;
       break;
 
@@ -423,16 +420,19 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       put_dos_date2(p,8,adate);
       put_dos_date2(p,12,mdate);
       SIVAL(p,16,size);
-      SIVAL(p,20,1024);
+      SIVAL(p,20,ROUNDUP(size,1024));
       SSVAL(p,24,mode);
       CVAL(p,32) = strlen(fname);
       strcpy(p + 33, fname);
-      name_ptr = p+33;
+      nameptr = p+33;
       p += 33 + strlen(fname) + 1;
       break;
 
     case SMB_FIND_FILE_BOTH_DIRECTORY_INFO:
-      SIVAL(p,0,94+strlen(fname)); p += 4;
+      was_8_3 = is_8_3(fname);
+      len = 94+strlen(fname);
+      len = (len + 3) & ~3;
+      SIVAL(p,0,len); p += 4;
       SIVAL(p,0,reskey); p += 4;
       put_long_date(p,cdate); p += 8;
       put_long_date(p,adate); p += 8;
@@ -443,20 +443,28 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       SIVAL(p,0,mode); p += 4;
       SIVAL(p,0,strlen(fname)); p += 4;
       SIVAL(p,0,0); p += 4;
-      if (!is_8_3(fname)) {
+      if (!was_8_3) {
+#ifndef KANJI
 	strcpy(p+2,unix2dos_format(fname,False));
-	name_map_mangle(p+2,True,SNUM(cnum));
+#else 
+	strcpy(p+2,fname);
+#endif
+	if (!name_map_mangle(p+2,True,SNUM(cnum)))
+	  (p+2)[12] = 0;
       } else
 	*(p+2) = 0;
       strupper(p+2);
       SSVAL(p,0,strlen(p+2));
       p += 2 + 24;
-      /* name_ptr = p;  */
+      /* nameptr = p;  */
       strcpy(p,fname); p += strlen(p);
+      p = pdata + len;
       break;
 
     case SMB_FIND_FILE_DIRECTORY_INFO:
-      SIVAL(p,0,64+strlen(fname)); p += 4;
+      len = 64+strlen(fname);
+      len = (len + 3) & ~3;
+      SIVAL(p,0,len); p += 4;
       SIVAL(p,0,reskey); p += 4;
       put_long_date(p,cdate); p += 8;
       put_long_date(p,adate); p += 8;
@@ -466,12 +474,15 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       SIVAL(p,0,size); p += 8;
       SIVAL(p,0,mode); p += 4;
       SIVAL(p,0,strlen(fname)); p += 4;
-      strcpy(p,fname); p += strlen(p);
+      strcpy(p,fname);
+      p = pdata + len;
       break;
       
       
     case SMB_FIND_FILE_FULL_DIRECTORY_INFO:
-      SIVAL(p,0,68+strlen(fname)); p += 4;
+      len = 68+strlen(fname);
+      len = (len + 3) & ~3;
+      SIVAL(p,0,len); p += 4;
       SIVAL(p,0,reskey); p += 4;
       put_long_date(p,cdate); p += 8;
       put_long_date(p,adate); p += 8;
@@ -482,14 +493,18 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       SIVAL(p,0,mode); p += 4;
       SIVAL(p,0,strlen(fname)); p += 4;
       SIVAL(p,0,0); p += 4;
-      strcpy(p,fname); p += strlen(p);
+      strcpy(p,fname);
+      p = pdata + len;
       break;
 
     case SMB_FIND_FILE_NAMES_INFO:
-      SIVAL(p,0,12+strlen(fname)); p += 4;
+      len = 12+strlen(fname);
+      len = (len + 3) & ~3;
+      SIVAL(p,0,len); p += 4;
       SIVAL(p,0,reskey); p += 4;
       SIVAL(p,0,strlen(fname)); p += 4;
-      strcpy(p,fname); p += strlen(p);
+      strcpy(p,fname);
+      p = pdata + len;
       break;
 
     default:      
@@ -506,7 +521,7 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
   }
 
   /* Setup the last_filename pointer, as an offset from base_data */
-  *last_name_off = PTR_DIFF(name_ptr,base_data);
+  *last_name_off = PTR_DIFF(nameptr,base_data);
   /* Advance the data pointer to the next slot */
   *ppdata = p;
   return(found);
@@ -614,6 +629,10 @@ static int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum
   /* a special case for 16 bit apps */
   if (strequal(mask,"????????.???")) strcpy(mask,"*");
 
+  /* handle broken clients that send us old 8.3 format */
+  string_sub(mask,"????????","*");
+  string_sub(mask,".???",".*");
+
   /* Save the wildcard match and attribs we are using on this directory - 
      needed as lanman2 assumes these are being saved between calls */
 
@@ -863,7 +882,7 @@ static int call_trans2qfsinfo(char *inbuf, char *outbuf, int length, int bufsize
   uint16 info_level = SVAL(params,0);
   int data_len;
   struct stat st;
-  char *sname = lp_servicename(SNUM(cnum));
+  char *vname = volume_label(SNUM(cnum));
   
   DEBUG(3,("call_trans2qfsinfo: cnum = %d, level = %d\n", cnum, info_level));
 
@@ -893,11 +912,11 @@ static int call_trans2qfsinfo(char *inbuf, char *outbuf, int length, int bufsize
     case 2:
     { 
       /* Return volume name */
-      int volname_len = MIN(strlen(sname),11);
+      int volname_len = MIN(strlen(vname),11);
       data_len = l2_vol_szVolLabel + volname_len + 1;
       put_dos_date2(pdata,l2_vol_fdateCreation,st.st_ctime);
       SCVAL(pdata,l2_vol_cch,volname_len);
-      StrnCpy(pdata+l2_vol_szVolLabel,sname,volname_len);
+      StrnCpy(pdata+l2_vol_szVolLabel,vname,volname_len);
       DEBUG(5,("call_trans2qfsinfo : time = %x, namelen = %d, name = %s\n",st.st_ctime,volname_len,
 	       pdata+l2_vol_szVolLabel));
       break;
@@ -910,14 +929,14 @@ static int call_trans2qfsinfo(char *inbuf, char *outbuf, int length, int bufsize
       PutUniCode(pdata+12,FSTYPE_STRING);
       break;
     case SMB_QUERY_FS_LABEL_INFO:
-      data_len = 4 + strlen(sname);
-      SIVAL(pdata,0,strlen(sname));
-      strcpy(pdata+4,sname);      
+      data_len = 4 + strlen(vname);
+      SIVAL(pdata,0,strlen(vname));
+      strcpy(pdata+4,vname);      
       break;
     case SMB_QUERY_FS_VOLUME_INFO:      
-      data_len = 17 + strlen(sname);
-      SIVAL(pdata,12,strlen(sname));
-      strcpy(pdata+17,sname);      
+      data_len = 17 + strlen(vname);
+      SIVAL(pdata,12,strlen(vname));
+      strcpy(pdata+17,vname);      
       break;
     case SMB_QUERY_FS_SIZE_INFO:
       {
@@ -970,7 +989,9 @@ static int call_trans2setfsinfo(char *inbuf, char *outbuf, int length, int bufsi
   reply to a TRANS2_QFILEINFO (query file info by fileid)
 ****************************************************************************/
 static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length, 
-			     int bufsize,int cnum,char **pparams,char **ppdata)
+				    int bufsize,int cnum,
+				    char **pparams,char **ppdata,
+				    int total_data)
 {
   char *params = *pparams;
   char *pdata = *ppdata;
@@ -987,7 +1008,7 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
 
 
   if (tran_call == TRANSACT2_QFILEINFO) {
-    int16 fnum = SVAL(params,0);
+    int16 fnum = SVALS(params,0);
     info_level = SVAL(params,2);
 
     CHECK_FNUM(fnum,cnum);
@@ -1013,8 +1034,8 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
   }
 
 
-  DEBUG(3,("call_trans2qfilepathinfo %s level=%d call=%d\n",
-	   fname,info_level,tran_call));
+  DEBUG(3,("call_trans2qfilepathinfo %s level=%d call=%d total_data=%d\n",
+	   fname,info_level,tran_call,total_data));
 
   p = strrchr(fname,'/'); 
   if (!p) 
@@ -1028,7 +1049,21 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
   
   params = *pparams = Realloc(*pparams,2); bzero(params,2);
   data_size = 1024;
-  pdata = *ppdata = Realloc(*ppdata, data_size); bzero(pdata,data_size);
+  pdata = *ppdata = Realloc(*ppdata, data_size); 
+
+  if (total_data > 0 && IVAL(pdata,0) == total_data) {
+    /* uggh, EAs for OS2 */
+    DEBUG(4,("Rejecting EA request with total_data=%d\n",total_data));
+#if 0
+    SSVAL(params,0,ERROR_EAS_NOT_SUPPORTED);
+    send_trans2_replies(outbuf, bufsize, params, 2, *ppdata, 0);
+    return(-1);
+#else
+    return(ERROR(ERRDOS,ERROR_EAS_NOT_SUPPORTED));
+#endif
+  }
+
+  bzero(pdata,data_size);
 
   switch (info_level) 
     {
@@ -1039,21 +1074,26 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
       put_dos_date2(pdata,l1_fdateLastAccess,sbuf.st_atime);
       put_dos_date2(pdata,l1_fdateLastWrite,sbuf.st_mtime);
       SIVAL(pdata,l1_cbFile,size);
-      SIVAL(pdata,l1_cbFileAlloc,1024); /* Make 1024 for now */
+      SIVAL(pdata,l1_cbFileAlloc,ROUNDUP(size,1024));
       SSVAL(pdata,l1_attrFile,mode);
+      SIVAL(pdata,l1_attrFile+2,4); /* this is what OS2 does */
       break;
 
     case 3:
-    case 4:
-      data_size = (info_level==3?24:28);
+      data_size = 24;
       put_dos_date2(pdata,0,sbuf.st_ctime);
       put_dos_date2(pdata,4,sbuf.st_atime);
       put_dos_date2(pdata,8,sbuf.st_mtime);
       SIVAL(pdata,12,size);
-      SIVAL(pdata,16,1024);
+      SIVAL(pdata,16,ROUNDUP(size,1024));
       SIVAL(pdata,20,mode);
       break;
 
+    case 4:
+      data_size = 4;
+      SIVAL(pdata,0,data_size);
+      break;
+
     case 6:
       return(ERROR(ERRDOS,ERRbadfunc)); /* os/2 needs this */      
 
@@ -1162,7 +1202,7 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
     return(ERROR(ERRSRV,ERRaccess));
 
   if (tran_call == TRANSACT2_SETFILEINFO) {
-    int16 fnum = SVAL(params,0);
+    int16 fnum = SVALS(params,0);
     info_level = SVAL(params,2);    
 
     CHECK_FNUM(fnum,cnum);
@@ -1203,6 +1243,16 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
   tvs.actime = st.st_atime;
   mode = dos_mode(cnum,fname,&st);
 
+  if (total_data > 0 && IVAL(pdata,0) == total_data) {
+    /* uggh, EAs for OS2 */
+    DEBUG(4,("Rejecting EA request with total_data=%d\n",total_data));
+    SSVAL(params,0,ERROR_EAS_NOT_SUPPORTED);
+
+    send_trans2_replies(outbuf, bufsize, params, 2, *ppdata, 0);
+  
+    return(-1);    
+  }
+
   switch (info_level)
     {
     case 1:
@@ -1213,8 +1263,6 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
       break;
 
     case 2:
-      if(IVAL(pdata,l2_cbList) || total_data>32)
-	return(ERROR(ERRDOS,ERROR_EAS_NOT_SUPPORTED));
       tvs.actime = make_unix_date2(pdata+l1_fdateLastAccess);
       tvs.modtime = make_unix_date2(pdata+l1_fdateLastWrite);
       mode = SVAL(pdata,l1_attrFile);
@@ -1229,8 +1277,6 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
       break;
 
     case 4:
-      if (IVAL(pdata,28) != 0 || total_data>36)
-	return(ERROR(ERRDOS,ERROR_EAS_NOT_SUPPORTED));	
       tvs.actime = make_unix_date2(pdata+8);
       tvs.modtime = make_unix_date2(pdata+12);
       size = IVAL(pdata,16);
@@ -1238,7 +1284,7 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
       break;
 
     case SMB_SET_FILE_BASIC_INFO:
-      pdata += 8; /* create time */
+      pdata += 8;		/* create time */
       tvs.actime = interpret_long_date(pdata); pdata += 8;
       tvs.modtime=MAX(interpret_long_date(pdata),interpret_long_date(pdata+8));
       pdata += 16;
@@ -1246,7 +1292,7 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
       break;
 
     case SMB_SET_FILE_END_OF_FILE_INFO:
-      if (IVAL(pdata,4) != 0) /* more than 32 bits? */
+      if (IVAL(pdata,4) != 0)	/* more than 32 bits? */
 	return(ERROR(ERRDOS,ERRunknownlevel));
       size = IVAL(pdata,0);
       break;
@@ -1257,6 +1303,7 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
       return(ERROR(ERRDOS,ERRunknownlevel));
     }
 
+
   if (!tvs.actime) tvs.actime = st.st_atime;
   if (!tvs.modtime) tvs.modtime = st.st_mtime;
   if (!size) size = st.st_size;
@@ -1267,7 +1314,7 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
     if(sys_utime(fname, &tvs)!=0)
       return(ERROR(ERRDOS,ERRnoaccess));
   }
-  if(mode != dos_mode(cnum,fname,&st) && dos_chmod(cnum,fname,mode)) {
+  if(mode != dos_mode(cnum,fname,&st) && dos_chmod(cnum,fname,mode,NULL)) {
     DEBUG(2,("chmod of %s failed (%s)\n", fname, strerror(errno)));
     return(ERROR(ERRDOS,ERRnoaccess));
   }
@@ -1401,7 +1448,7 @@ int reply_findclose(char *inbuf,char *outbuf,int length,int bufsize)
 {
   int cnum;
   int outsize = 0;
-  uint16 dptr_num=SVAL(inbuf,smb_vwv0);
+  int16 dptr_num=SVALS(inbuf,smb_vwv0);
 
   cnum = SVAL(inbuf,smb_tid);
 
@@ -1423,7 +1470,7 @@ int reply_findnclose(char *inbuf,char *outbuf,int length,int bufsize)
 {
   int cnum;
   int outsize = 0;
-  int dptr_num=-1;
+  int dptr_num= -1;
 
   cnum = SVAL(inbuf,smb_tid);
   dptr_num = SVAL(inbuf,smb_vwv0);
@@ -1442,6 +1489,15 @@ int reply_findnclose(char *inbuf,char *outbuf,int length,int bufsize)
 }
 
 
+/****************************************************************************
+  reply to a SMBtranss2 - just ignore it!
+****************************************************************************/
+int reply_transs2(char *inbuf,char *outbuf,int length,int bufsize)
+{
+  DEBUG(4,("Ignoring transs2 of length %d\n",length));
+  return(-1);
+}
+
 /****************************************************************************
   reply to a SMBtrans2
 ****************************************************************************/
@@ -1499,14 +1555,12 @@ int reply_trans2(char *inbuf,char *outbuf,int length,int bufsize)
     /* We need to send an interim response then receive the rest
        of the parameter/data bytes */
       outsize = set_message(outbuf,0,0,True);
-      send_smb(outbuf);
+      send_smb(Client,outbuf);
 
       while( num_data_sofar < total_data || num_params_sofar < total_params)
 	{
-	  receive_smb(inbuf, 0);
-	  
-	  /* Ensure this is still a trans2 packet (sanity check) */
-	  if(CVAL(inbuf, smb_com) != SMBtrans2)
+	  if(!receive_smb(Client,inbuf, SMB_SECONDARY_WAIT) ||
+	     CVAL(inbuf, smb_com) != SMBtranss2)
 	    {
 	      outsize = set_message(outbuf,0,0,True);
 	      DEBUG(2,("Invalid secondary trans2 packet\n"));
@@ -1552,7 +1606,7 @@ int reply_trans2(char *inbuf,char *outbuf,int length,int bufsize)
       break;
     case TRANSACT2_QPATHINFO:
     case TRANSACT2_QFILEINFO:
-      outsize = call_trans2qfilepathinfo(inbuf, outbuf, length, bufsize, cnum, &params, &data);
+      outsize = call_trans2qfilepathinfo(inbuf, outbuf, length, bufsize, cnum, &params, &data, total_data);
       break;
     case TRANSACT2_SETPATHINFO:
     case TRANSACT2_SETFILEINFO:
